


;;   FILE:= MODEM12.ASM            CREATED 10-20-81
;;------------------------------------------------------------------------
;;  Smartmodem 1200 firmware
;;  SOURCE VERSION 4.7  , EPROM VERSION 3
;;  LAST REV. 1-27-83  (after beta test)
;;
;;  WRITTEN BY Dale Heatherington
;;
;;  Copyright 1982 by Hayes Microcomputer Products, Inc.
;;-------------------------------------------------------------------------
;;**** THIS HAS BECOME THE SMARTMODEM 1200 SOURCE FILE
;;**** AS OF 2-10-82.
;;
;; THIS IS FOR A 7.9872 MHZ CRYSTAL
;; CYCLE TIME IS 250.40 NS.
;;---------------------------------------------------
;;      212A DIBIT ENCODING
;;LEFT BIT IS SENT/RECEIVED FIRST
;;      00      =       +90 DEG.
;;      01      =       0 DEG.
;;      10      =       180 DEG.
;;      11      =       -90 DEG.
;;---------------------------------------------------
;;
;;      REGISTER MAP
;;
        PORT0:   EQU    0       ;; I/O PORT 0
        PORT1:   EQU    1       ;; I/O PORT 1
        PORT2:   EQU    2       ;; I/O PORT 2
        PORT3:   EQU    3       ;; I/O PORT 3
        SPARE:  EQU     4       ;; 6 MORE REGISTERS
        BUFFER:  EQU    10      ;; 40 CHARACTER INPUT BUFFER
        STACK:   EQU    40H     ;; TOP OF 20 BYTE STACK
        GLOBALS: EQU    40H     ;; GLOBAL VARIABLES 32 BYTES
        PRIMREG: EQU    60H     ;; PRIMARY REGISTER SET    16 BYTES
        INTREG:  EQU    70H     ;; INTERRUPT REGISTER SET  16 BYTES
;;
;;-------------------------------------------------------------------
;;      FREQUENCY LIST FOR 7.9872 MHZ CPU CRYSTAL
;;      WHICH IS DIVIDED DOWN TO 998.4 KHZ INTERNALLY
;;      IN THE Z8.  THE 998.4 KHZ IS DIVIDED BY 104 BY
;;      A Z8 INTERNAL TIMER-COUNTER TO YIELD 9600 HZ
;;      MAIN INTERRUPT RATE.  DIVISION BY 8 GIVES 1200 BAUD
;;      FOR THE LINE RATE.
;;      F * .853333 = THE FOLLOWING CONSTANTS.
;;
        MASTER: EQU     104
;;
        F697:   EQU     595*4
        F770:   EQU     657*4
        F852:   EQU     727*4
        F941:   EQU     803*4
        F1070:  EQU     913*4
        F1200:  EQU     1024*4
        F1209:  EQU     1032*4
        F1270:  EQU     1084*4
        F1336:  EQU     1140*4
        F1477:  EQU     1260*4
        F1633:  EQU     1393*4
        F2025:  EQU     1728*4
        F2225:  EQU     1899*4
        F2400:  EQU     2048*4
;;-------------------------------------------------------
;; EXTENDED I/O EQUATES
;;
;;INPUT BITS:
;; LOAD PORT 0 WITH THE FOLLOWING CONSTANTS THEN READ P3-3
;;
        DTR:    EQU     0
        SW2:    EQU     10H
        SW3:    EQU     20H
        SW4:    EQU     30H
        SW5:    EQU     40H
        SW8:    EQU     50H
        CARDET: EQU     80H
        DIBIT1: EQU     90H
        DIBIT2: EQU     0A0H
        RXFSK:  EQU     0B0H
;;
;;
;;--------------------------------------------------------


        
        DAC:    EQU     2
;;
;;THESE ARE THE "S" REGISTERS
;;
SR:     EQU     GLOBALS
RINGN:  EQU     SR              ;;RING TO ANSWER ON
RINGS:  EQU     SR+1            ;;RINGS WHICH HAVE OCCURED
ESCODE: EQU     SR+2            ;;ESCAPE CHARACTER
CRC:    EQU     SR+3            ;;CARRAGE RETURN CHAR.
LFC:    EQU     SR+4            ;;LINE FEED CHAR.
BSC:    EQU     SR+5            ;;BACK SPACE CHAR.
DTW:    EQU     SR+6            ;;DIAL TONE WAIT TIME
CDW:    EQU     SR+7            ;;CARRIER DETECT WAIT TIME
PT:     EQU     SR+8            ;;PAUSE TIME FOR COMMA
CDELAY: EQU     SR+9            ;;CARRIER DETECT RESPONSE TIME
LCDT:   EQU     SR+10           ;;LOST CARRIER TO DISCONNECT TIME
TONLEN: EQU     SR+11           ;;DTMF TONE DURATION AND SPACING
EGT:    EQU     SR+12           ;;ESCAPE CODE GUARD TIME
USTAT:  EQU     SR+13           ;;UART STATUS REGISTER
OFLAGS: EQU     SR+14           ;;OPTION FLAGS
PFLAGS: EQU     SR+15           ;;PROGRAM STATUS FLAGS
LPBACK: EQU     SR+16           ;;SELF TEST CONTROL BYTE...
                                ;;BIT 0=ANALOG LOOP BACK
                                ;;BIT 1=LONG TOUCH TONE TEST MODE
;;
;; END OF PUBLISHED "S" REGISTERS
;;
;; BIT MAP FOR USTAT
;;
RRF:    EQU     1               ;; 1=RECEIVE REGISTER FULL
EXR:    EQU     2               ;; 1=EXTENDED RESULT CODES
PE:     EQU     4               ;;PARITY ENABLE
EPE:    EQU     8               ;;EVEN PARITY ENABLE
DBITS:  EQU     10H             ;; 1=8 DATA BITS  0=7 DATA BITS
IBF:    EQU     20H             ;;INPUT BUFFER FULL FLAG
OVL:    EQU     40H             ;;INPUT BUFFER OVERFLOW FLAG
DB8:    EQU     80H             ;;STUCK PARITY STATE OF DATA BIT 8
;;
;; BIT MAP FOR OFLAGS
;;
AAON:   EQU     1               ;;AUTO ANSWER ON
LEON:   EQU     2               ;;LOCAL ECHO ON
QUIET:  EQU     4               ;;TRUE = RESULT CODES OFF
VBON:   EQU     8               ;;VERBOSE MODE ON
STUPID: EQU     10H             ;;1=DONT EXECUTE ANY COMMANDS
TDIAL:  EQU     20H             ;;1=TOUCH TONE  0=PULSE
SPON:   EQU     40H             ;;SPEAKER ON UNTIL CARRIER
SPON2:  EQU     80H             ;;SPEAKER ON ALWAYS
;;
;; BITMAP FOR PFLAGS
;;
IBAUD0: EQU     1               ;;INPUT BAUD RATE SELECT BIT 0
IBAUD1: EQU     2               ;;INPUT BAUD RATE SELECT BIT 1
MODE:   EQU     4               ;;1=ORIG   0=ANSW
DPLX:   EQU     8               ;;1=FULL DUPLEX   0=HALF DUPLEX
OBAUD0: EQU     10H             ;;OUTPUT BAUDRATE SELECT BIT 0
OBAUD1: EQU     20H             ;;OUTPUT BAUDRATE SELECT BIT 1
TXON:   EQU     40H             ;;1=MODEM CARRIER ENABLED
ONLINE: EQU     80H             ;;1=ON-LINE   0=LOCAL COMMAND MODE
;;GLOBAL REGISTER EQUATES
;;
;;
IRQ5VH: EQU     SPARE           ;;DESTINATION FOR TIMER 1 INTERRUPT
IRQ5VL: EQU     SPARE+1
DSH:    EQU     SPARE+2         ;;DE-SCRAMBLER REGISTER
DSM:    EQU     SPARE+3
DSL:    EQU     SPARE+4
COUNTER: EQU    SPARE+5



GL:     EQU     51H
IMAGE35:EQU     GL
IMAGE34:EQU     GL+1            ;;COPY OF EXTERNAL CONTROL BITS
CHARS:  EQU     GL+2            ;;NUMBER OF CHARACTERS IN BUFFER
CHAR1:  EQU     GL+3            ;;FIRST CHARACTER RECEIVED FOR BAUD RATE
RXHOLD: EQU     GL+4            ;;RECEIVER HOLDING REGISTER
TXHOLD: EQU     GL+5            ;;TRANSMITTER HOLDING REGISTER
CT1200: EQU     GL+6            ;;COUNTER, DECREMENTED AT 1200 HZ RATE
IRQ3PTR:EQU     GL+7
IRQ4PTR:EQU     GL+8
BITCOUNT: EQU   GL+9            ;;BIT COUNTER FOR RX UART ROUTINE
LCHARS: EQU     GL+10           ;;LENGTH OF LAST BUFFER (FOR REPEAT)
TXDATA: EQU     GL+11
TXFLAG: EQU     GL+12           ;;FF=NORMAL XMIT CONTROL, (NOT TXON)=NO CARRIER
RESULTS: EQU    GL+13           ;;RESULT CODE DIGIT 0..5
BUFPNTR: EQU    GL+14           ;;RELATIVE BUFFER POINTER FOR COMMANDS
;;
FLAGS:  EQU     252             ;;Z8 STATUS FLAG REGISTER
ADDRESS: EQU    0FEH            ;;USED BY "S" COMMANDS
;;
MAXCHARS:       EQU     40      ;;LENGTH OF INPUT BUFFER
;;
;;MAP OF IMAGE35:
SWHOOK: EQU     1               ;;1=OFF HOOK
RJ13:   EQU     2               ;;1=RJ13 AUX CONTACTS CLOSED
AA$LED: EQU     4               ;;0=AA LED ON
SPEAKER:EQU     8               ;;1=SPEAKER ON
HS$LED: EQU     10H             ;;0=HS LED ON
RING:   EQU     20H             ;;0=RING INDICATOR TRUE
CARRIER:EQU     40H             ;;0=CARRIER DETECT TRUE
F1200HZ:EQU     80H             ;;1=PASS 1200 HZ  0=PASS 2400 HZ
;;
;;MAP OF IMAGE 34
;;
VCO:    EQU     1               ;;0=VCO ON
PSK:    EQU     2               ;;1=PSK MODE  0=FSK MODE
ANSW:   EQU     4               ;;1=ANSWER MODE
LOFREQ: EQU     8               ;;1=RECV 2400 HZ  0=RECV 2125 HZ
TCA:    EQU     80H             ;;INTERNAL FLAG, TX CHAR. AVALABLE
;;
;;
;;INTERRUPT VECTORS
;;
        DW      DATACLK         ;;IRQ0 P3-2 -CLOCK (600 HZ)
        DW      XIRET           ;;IRQ1 P3-3 INPUT FROM U5 MUX (NOT USED)
        DW      DIBIT           ;;IRQ2 P3-1 +CLOCK (600 HZ) (DATA SAMPLED)
        DW      IRQ3            ;;IRQ3 P3-0 TX DATA
        DW      IRQ4            ;;IRQ4 VECTOR FOR TIMER 0
        DW      IRQ5            ;;IRQ5 VECTOR FOR TIMER 1
;;
        JP      START           ;;GO FOR IT!!
;;
IRQ5:   JP      @IRQ5VH         ;;MAIN TIME BASE ROUTINE
;;
STATE1: LD      R3,#STATE3&255
        SRP     PRIMREG
XIRET:  IRET
;;
STATE3: LD      R3,#STATE5&255
ENTRS3: OR      CT1200,CT1200           ;;TEST COUNTER FOR ZERO
        JR      Z,S3DONE
        DEC     CT1200                  ;;DECREMENT IF NOT ZERO
S3DONE: SRP     PRIMREG
        IRET
;;      
;;
;;DATA SCRAMBLER ROUTINE.   INTERRUPT REGISTERS R5,R6 & R7
;; FORM A 17 BIT SHIFT REGISTER.  R5 IS MSB, R7 IS LSB
;;
STATE5: ;;THIS ROUTINE EXECUTED ON BOTH STATE 5 AND STATE D.
        LD      R3,#STATE7&255
SCRAMB: RRC     R5              ;; BIT 17 OF SCRAMBLER INTO CARRY
        TM      R6,#20H         ;;TEST BIT 14
        JR      Z,SC0
        CCF                     ;;EXOR 17 AND 14
SC0:    TM      TXDATA,#1       ;;TEST DATA BIT
        JR      Z,SC1
        CCF                     ;;EXOR WITH DATA
SC1:    ADC     R7,R7           ;;SHIFT THE SCRAMBLER REGISTER
        ADC     R6,R6
        ADC     R5,R5
SC2:    SRP     PRIMREG
        IRET
;;
STATE7: LD      R3,#STATE9&255
        SRP     PRIMREG
        IRET
;;
STATE9:
        LD      PORT0,#CARDET           ;;ADDRESS HARDWARE CARRIER DETECT BIT
        AND     IMAGE35,#"CARRIER       ;;ASSUME CARRIER
        TM      PORT3,#8                ;;TEST FOR CARRIER
        JR      NZ,S9DONE               ;;EXIT WITH CARRIER TRUE
        OR      IMAGE35,#CARRIER        ;; OR FALSE
S9DONE: LD      R3,#STATEB&255
        SRP     PRIMREG
        IRET
;;      
;;
STATEB: LD      R3,#STATED&255
        JR      ENTRS3                  ;;1200 HZ COUNTER
;;
;;
STATED: LD      R3,#STATEF&255
        JR      SCRAMB
;;
;;
STATEF: 
        LD      R3,R7                   ;;GET DIBITS FROM SCRAMBLER
        AND     R3,#3                   ;;KEEP LOW 2 BITS
        ADD     R3,R3                   ;;MPY BY 2
        ADD     R3,#DBENC &255          ;;R2,R3 =POINTER TO DIBIT TABLE
        LDC     R8,@RR2                 ;;GET HIGH BYTE OF MODULATION ADR.
        INC     R3                      ;;POINT TO LOW BYTE
        LDC     R9,@RR2                 ;;GET LOW BYTE OF MODULATION ADDRESS
        LD      R3,#STATE1 & 255        ;;RESET THE STATE POINTER
        SRP     PRIMREG
        IRET

;;
;;

;;
;;------------------------------------------------------
;;212A DIBIT ENCODING TABLE
;; RIGHT MOST BIT SENT FIRST.
DBENC:
        DW      PL90S0  ;;DIBIT 00 = +90 DEG.
        DW      ZERO    ;;DIBIT 01 = 0
        DW      PLUS180 ;;DIBIT 10 = 180
        DW      MI90S0  ;;DIBIT 11 = -90
;;

;;------------------------------------------------------
;;THIS TABLE IS USED BY THE PARITY ROUTINE THE SET
;;PROPER STATUS BITS IN "USTAT"
;;
FORMAT: DB      DBITS           ;;8 DATA, NO PARITY
        DB      EPE+PE          ;;7 DATA, EVEN PARITY
        DB      0               ;;7 DATA, PARITY STUCK AT 0
        DB      DB8+DBITS       ;;8 DATA, NO PARITY
        DB      DBITS+EPE+PE    ;;8 DATA, EVEN PARITY
        DB      DBITS+EPE+PE
        DB      DBITS+EPE+PE
        DB      DBITS+EPE+PE+DB8
        DB      DBITS+PE        ;;8 DATA, ODD PARITY
        DB      DB8+DBITS+PE
        DB      DB8+DBITS+PE
        DB      DB8+DBITS+PE
        DB      DBITS           ;; 8 DATA, NO PARITY
        DB      EPE+PE          ;; 7 DATA, EVEN PARITY
        DB      PE              ;; 7 DATA, ODD PARITY
        DB      DB8             ;; 7 DATA, 8TH BIT MARK
;;
P10TAB:
        DB      100
        DB      10
;;THE LAST ENTRY IS THE FIRST ENTRY IN THE NEXT TABLE!
;;
;;THIS DATA GOES INTO THE "S" REGISTERS AT COLD START
;;
INITDATA:
        DB      1               ;;RING TO ANSWER ON
        DB      0               ;;RINGS
        DB      '+'             ;;ESCAPE CHARACTER
        DB      0DH             ;;<CR>
        DB      0AH             ;;<LF>
        DB      8               ;;BACK SPACE
        DB      2               ;;DIAL TONE WAIT TIME
        DB      30              ;;CARRIER DETECT WAIT TIME
        DB      2               ;;COMMA PAUSE TIME
        DB      6               ;;CARRIER DETECT RESP. TIME (X100 MS)
        DB      7               ;;LOST CARRIER TO HANG UP DELAY (X100 MS)
        DB      70              ;;DTMF DURATION
        DB      50              ;;ESCAPE CODE GUARD TIME
        DB      DBITS           ;;USTAT
        DB      SPON+VBON       ;;OFLAGS
        DB      OBAUD0+OBAUD1+DPLX ;;PFLAGS
        DB      0               ;;LOOP BACK CONTROL
;;      
;;
;;
;;
RESTART:
        LD      RESULTS,#'0'    ;;COME HERE AFTER Z COMMAND <-----
        CALL    RESULT
;;
START:  DI                      ;;COME HERE AFTER COLD START <-----
        CLR     PORT0           ;;PRESET PORT0 TO AVOID GITCHING ROM ADDRESS
        LD      P01M,#16H       ;;PORT 1 = ADR/DATA / PORT 0 = OUTPUT/ADR
        LD      PORT3,#0F0H     ;;PRESET PORT3 TO AVOID GLITCHING RS232 DATA
        LD      P3M,#1          ;;SIMPLE I/O MODE FOR PORT 3
        LD      PORT2,#80H      ;;PRESET DAC FOR ZERO VOLTS
        LD      P2M,#0          ;;ALL OUTPUTS FOR PORT 2
        LD      SPH,#4          ;;USE SPH AS POINTER FOR CLEAR OPERATION
LL000:  CLR     @SPH            ;;CLEAR ALL REGISTERS 4-7F
        INC     SPH             ;;   /
        JR      PL,LL000        ;;  /
;;
        LD      SPL,#STACK      ;;SET STACK POINTER
        SRP     PRIMREG         ;;INITIALIZE THE "S" REGISTERS
        CLR     R14
        LD      R15,#INITDATA & 255
        LD      R12,#SR         ;;POINT R12 TO S REGISTERS
        LD      R13,#17         ;;NUMBER OF REGISTERS TO INIT.
LL001A: LDCI    @R12,@RR14      ;;DO INIT.
        DJNZ    R13,LL001A
;;
;;INITIALIZE COUNTERS-TIMERS
;;
        LD      IPR,#0BH
        LD      IMR,#20H                ;;ENABLE IRQ5 
        LD      PRE1,#7                 ;;MOD N, INTERNAL, DIVIDE BY 1
        LD      T1,#MASTER              ;;MASTER 9600 HZ CLOCK (IRQ5)
        LD      TMR,#0CCH               ;;INT. CLK OUT, START T1
        LD      IRQ5VH,#TIMEBASE > 8
        LD      IRQ5VL,#TIMEBASE & 255  ;;SET T1 INTERRUPT DESTINATION
        CALL    ZERO$FREQ               ;;KILL TRANSMITTER
        LD      TXFLAG,#TXON            ;; "C1" COMMAND DEFAULT
        EI
        CALL    WAIT250                 ;;WAIT 250 MS FOR RESET
        DI
;;
;;NOW READ IN THE OPTION SWITCHES
;;
        CLR     OFLAGS          ;;ZERO OPTION FLAG REGISTER
        LD      R15,#40H        ;;ADDRESS OF "AA" SWITCH 5
LL002:  LD      PORT0,R15       ;;SEND OUT THE ADDRESS TO THE MUX
        NOP                     ;;WAIT FOR MUX PROP DELAY
        LD      R14,PORT3       ;;BIT 3 OF PORT 3 IS THE DATA BIT WE WANT
        AND     R14,#00001000B  ;;KEEP ONLY BIT 3
        RR      OFLAGS          ;;SHIFT OVER THE OTHER BITS
        OR      OFLAGS,R14      ;;PUT IN THE NEW BIT
        SUB     R15,#10H        ;;NEXT ADDRESS
        JR      NZ,LL002        ;;LOOP 4 TIMES FOR AA,ECHO,QUIET,VERBOSE
        LD      PORT0,#50H      ;;NOW ADDRESS SWITCH 8
        NOP
        LD      R14,PORT3       ;;GET IT AT BIT 3
        RLC     R14             ;;MOVE IT 1 BIT LEFT TO BIT 4
        AND     R14,#00010000B  ;;KEEP ONLY THAT BIT
        OR      OFLAGS,R14      ;;PUT IN IN BIT 4 OF "OFLAGS"
        OR      OFLAGS,#SPON    ;;DEFAULT TO SPEAKER ON TIL CARRIER
        TM      OFLAGS,#AAON    ;;TEST FOR AUTO-ANSWER
        JR      NZ,LL002A       ;;JUMP IF SET TO AUTO-ANSWER
        CLR     RINGN           ;;ELSE CLEAR THE S0 REGISTER
LL002A: CLR     IMAGE35
        TM      OFLAGS,#AAON    ;;WANT TO AUTO ANSWER?
        JR      NZ,LL002B       ;;JUMP IF WE DO
        OR      IMAGE35,#AA$LED ;;ELSE TURN OFF AA LED
LL002B: OR      IMAGE35,#F1200HZ+CARRIER+RING
        LD      IMAGE34,#VCO+ANSW  ;;SET FOR RING DETECTION
        CALL    UD3435          ;;UPDATE 34 & 35
        CALL    CD$OFF          ;;CARRIER DETECT LED OFF
;;
OK:     LD      RESULTS,#'0'            ;;"OK" RESULT CODE
GETCOMM:
        DI
        AND     PFLAGS,#"ONLINE         ;;CLEAR ONLINE FLAG
        LD      SPL,#STACK              ;;SET STACK POINTER
        SRP     PRIMREG
        LD      IMR,#20H                ;;ONLY MASTER CLOCK INTERRUPT
        AND     TMR,#"3                 ;;STOP TIMER T0
        LD      PORT3,#0F0H             ;;SET RS232 DATA OUT TO MARK
        AND     IRQ,#20H                ;;CLEAR ALL OTHER INTERRUPT REQUEST
        EI
        CLR     IRQ3PTR                 ;;CLEAR START BIT INTERRUPT POINTER
        CLR     IRQ4PTR                 ;;CLEAR TIMER 0 INT. POINTER
        LD      LCHARS,CHARS            ;;SAVE LAST BUFFER LENGTH
        LD      CHARS,#-2
        AND     USTAT,#"(IBF+OVL)       ;;CLEAR BUFFER FULL FLAG
        CALL    CLRRI                   ;;CLEAR RING INDICATOR FLAG
        TM      PFLAGS,#IBAUD0+IBAUD1   ;;IS BAUD RATE KNOWN?
        JR      Z,TU0                   ;;JUMP IF BOTH BITS ARE ZERO
        CALL    RESULT                  ;;SEND RESULT CODE OF LAST COMMAND
TU0:    DI
        OR      IMR,#20H+8              ;;ENABLE UART RECEIVER
        EI
        CLR     R11                     ;;CLEAR 300 MS RING TIMER
        CLR     R10                     ;;...AND 8 SECOND RING ABORT TIMER
        TM      IMAGE35,#SWHOOK         ;;SEE IF WE ARE OFF-HOOK
        JR      NZ,TU2                  ;;BYPASS RING DETECTOR SET UP
        LD      IMAGE34,#5              ;;ANSWER MODE WITH VCO OFF
        CALL    UPDATE34                ;;SET UP FOR RING DETECTION
LOOP:   TM      IMAGE35,#SWHOOK         ;;TEST FOR OFF-HOOK CONDITION
        JR      NZ,TU1                  ;;DONT DO RING STUFF IF OFF-HOOK
        CP      R11,#6                  ;;CHECK FOR 300 MS OF RING SIGNAL
        JR      NE,TU2B                 ;;
        CALL    SETRI                   ;;SET RI AND AA-LED
TU2B:   TM      IMAGE35,#CARRIER        ;;SEE IF PHONE IS RINGING
        JR      NZ,TU2A                 ;;JUMP IF RING NOT IN PROGRESS
        CP      R11,#6
        JR      ULT,TU1                 ;;JUMP IF RING NOT ON FOR 300 MS
        LD      R10,#80                 ;;SET TIMEOUT TO 8 SEC. ON VALID RING
        JR      TU1
TU2A:   CP      R11,#6                  ;;SEE IF RING WAS PRESENT FOR 300 MS
        CLR     R11
        JR      ULT,TU1                 ;;JUMP IF NO VALID RING
;;
TU2C:   CALL    CLRRI                   ;;CLEAR RI AND AA-LED
        LD      RESULTS,#'2'            ;;GET RING RESULT CODE
        DI
        CP      IRQ3PTR,#0
        JR      NZ,TU2D                 ;;DONT SEND RING IF IN COMMAND LINE
        PUSH    IMR                     ;;SAVE INTERRUPT MASK REGISTER
        AND     IMR,#"18H               ;;KILL COMMAND PROCESSOR INTERRUPTS
        EI
        CALL    RESULT                  ;;SEND RING RESULT
        DI                              ;;     /
        POP     IMR                     ;;RE-ENABLE COMMAND PROCESSOR
TU2D:   LD      PORT0,#DTR              ;;    /**BUG FIX** REV 2
        NOP
        TM      PORT3,#8                ;;  /TEST FOR TRUE RS232 DTR LEAD
        EI                              ;; /
        JR      NZ,TU1                  ;;/ JUMP IF DTR FALSE (IGNORE RING)
        CP      RINGN,#0                ;;SEE IF SET TO IGNORE RINGS
        JR      EQ,TU1                  ;;JUMP IF WE IGNORE THE RING
        LD      R10,#(20*8)             ;;ELSE RESET 8 SECOND TIMER
        INC     RINGS                   ;;AND COUNT THE RING
        CP      RINGS,RINGN             ;;COMPARE RINGS TO STORED VALUE
        JP      EQ,A$COMM               ;; GO ONLINE IN ANSWER MODE IF MATCH
TU1:    CP      CT1200,#0               ;;CHECK 50 MS TIMER
        JR      NZ,TU2
        LD      CT1200,#60              ;;RELOAD AT ZERO
        INC     R11                     ;;R11 USED FOR 300 MS RING TIMER
        JR      NZ,TU1A
        DEC     R11                     ;;HOLD AT COUNT 255
TU1A:   DJNZ    R10,TU2                 ;;AND COUNT 50 MS INTERVALS
        CLR     RINGS                   ;;CLEAR RING COUNTER IF 8 SECONDS UP
        INC     R10
;;
TU2:    TM      OFLAGS,#LEON            ;;TEST FOR LOCAL ECHO
        JR      Z,TU3                   ;;JUMP IF NO LOCAL ECHO
        LD      R0,PORT3                ;;GET INPUT RS232 DATA (LSB)
        RR      R0                      ;;MOVE LSB TO MSB
        OR      R0,#70H
        LD      PORT3,R0                ;;ECHO THE BIT
TU3:    TM      IMAGE34,#VCO            ;;SEE IF WE ARE ONLINE
        JR      NZ,TU4A                 ;;JUMP IF NOT
        TM      IMAGE35,#CARRIER
        JR      Z,TU4A                  ;;JUMP IF CARRIER PRESENT
        CP      LCDT,#255               ;;SHOULD WE IGNORE CARRIER DETECT?
        JR      EQ,TU4A                 ;;  / YES, JUMP 
        CALL    SHOFF                   ;; /  ELSE HANGUP WHEN CARRIER LOST
        CLR     R11                     ;;/   AND CLEAR 300 MS RING TIMER 
TU4A:
        TM      USTAT,#RRF              ;;CHECK FOR CHARACTER
        JR      Z,TU4                   ;;JUMP IF NO CHARACTER PRESENT
        AND     USTAT,#"RRF             ;;ELSE CLEAR RRF FLAG
        CP      RXHOLD,BSC              ;;SEE IF BACK SPACE CHAR
        JR      NZ,TU4                  ;;JUMP IF NOT BACK SPACE
        CP      CHARS,#0                ;;ARE WE AT BEGINNING OF BUFFER?
        JR      LE,TU4                  ;;DONT BACK UP PAST BEGINNING
        DEC     CHARS                   ;;NOW BACK UP CHARACTER COUNTER
        LD      TXHOLD,#20H             ;;SEND A SPACE
        CALL    CHAROUT
        LD      TXHOLD,BSC              ;;THEN A BACK SPACE
        CALL    CHAROUT
TU4:
        TM      USTAT,#IBF              ;;INPUT BUFFER FULL?
        JP      Z,LOOP
        OR      IMAGE35,#HS$LED         ;;ASSUME LOW SPEED
        TCM     PFLAGS,#OBAUD0+OBAUD1   ;;TEST FOR 1200 BAUD
        JR      NZ,TU5
        XOR     IMAGE35,#HS$LED         ;;TURN ON HIGH SPEED LED
TU5:    CALL    UPDATE35
        CLR     BUFPNTR
        JR      EXECUTE
;;--------------------------------------------------------
;; THESE ROUTINES SET OR CLEAR THE AA-LED AND RING INDICATOR RS232 PIN
;; 4 BYTES OF STACK ARE USED
;; R0,R1,R2,PORT0,PORT3 ARE DISTURBED
;;
CLRRI:  OR      IMAGE35,#(AA$LED+RING)  ;;RI AND AA-LED OFF
        JR      SETRI0  
SETRI:  AND     IMAGE35,#"(AA$LED+RING) ;;RI AND AA-LED ON
SETRI0: CP      RINGN,#0                ;;SEE IF ENABLED TO ANSWER
        JR      EQ,TRICK1               ;; / IF NOT, DO HARWARE UPDATE NOW
        XOR     IMAGE35,#AA$LED         ;; / ELSE INVERT AA-LED STATE
TRICK1: JP      UPDATE35                ;;/  THEN UPDATE HAREDWARE
;;
;;--------------------------------------------------------------------
;;              ** COMMAND DECODER **
;;
EXECUTE:LD      SPL,#STACK              ;;RESET STACK POINTER
        LD      RESULTS,#'4'            ;;ASSUME ERROR RESULT
        TCM     PFLAGS,#IBAUD0+IBAUD1   ;;TEST FOR 1200 BAUD INPUT
        JR      NZ,EX00                 ;;JUMP IF LOW SPEED
        TCM     USTAT,#PE+DBITS         ;;TEST FOR 8 DATA BITS PLUS PARITY
        JR      Z,TRICK2                ;;ERROR! 1200 BAUD CANT HAVE 11 BITS
EX00:   TM      USTAT,#OVL              ;;CHECK FOR BUFFER OVERFLOW
        JP      NZ,GETCOMM              ;;QUIT NOW IF OVERFLOW
        CALL    NEXTCHR                 ;;GET NEXT CHARACTER IN BUFFER
        JP      C,OK                    ;;ALL DONE NOW IF CARRY SET
EX0:    LD      R8,#CMDTBL-3 > 8
        LD      R9,#CMDTBL-3 & 255      ;;POINT TO COMMAND TABLE
EX1:    ADD     R9,#3                   ;;ADVANCE POINTER
        ADC     R8,#0
        LDC     R6,@RR8                 ;;GET A TABLE ENTRY TO R6
        CP      R6,#';'                 ;;END OF TABLE MARKER IS ";"
TRICK2: JP      EQ,GETCOMM              ;;QUIT IF AT END WITH ERROR MSG
        CP      R6,R4                   ;;DO WE HAVE A MATCH?
        JR      NE,EX1                  ;;JUMP BACK IF NOT
FOUND:  INCW    RR8                     ;;POINT TO ADDRESS OF ROUTINE
        LDC     R6,@RR8                 ;;GET HIGH ADDRESS BYTE
        INCW    RR8
        LDC     R7,@RR8                 ;;GET LOW ADDRESS BYTE
        CALL    @RR6                    ;;CALL THE ROUTINE
        JR      EXECUTE                 ;;LOOP BACK FOR MORE COMMANDS
;;
;;
NEXTCHR:                        ;;RETURNS NEXT BUFFER CHR IN R4
                                ;;CARRY SET IF PAST END OF BUFFER
        LD      R5,BUFPNTR              ;;GET RELATIVE BUFFER POINTER
        LDRX    R4,R5,BUFFER            ;;GET THE DATA FROM THE BUFFER
        CP      CHARS,BUFPNTR           ;;AT END OF BUFFER??
        SCF
        JR      EQ,RETURN
        INC     BUFPNTR                 ;;ADVANCE BUFFER POINTER
        RCF
RETURN: RET
;;
;;
NEXTNUM:                        ;;CONVERTS NEXT DIGITS IN BUFFER
                                ;;TO BINARY VALUE IN R4
        CLR     R6
        CALL    NEXTCHR         ;;GET A DIGIT (THIS IS A FAKE OUT)
        JR      C,LL011         ;;QUIT IF END OF BUFFER
        DEC     BUFPNTR         ;;CORRECT BUFFER POINTER
LL008:  CALL    NEXTCHR         ;;REALLY GET A DIGIT NOW
        JR      C,LL010         ;;QUIT IF END OF BUFFER
        CP      R4,#'0'         ;;SEE IF ITS REALLY A DIGIT
        JR      ULT,LL009       ;;QUIT IF NOT
        CP      R4,#3AH         ;;ANOTHER TEST FOR DIGIT
        JR      UGE,LL009       ;;QUIT IF NOT DIGIT
        AND     R4,#0FH         ;;MASK ASCII BIAS BITS
R6X10:  LD      R5,R6           ;;MPY R6 TIMES 10
        ADD     R6,R6
        ADD     R6,R6
        ADD     R6,R6
        ADD     R6,R5
        ADD     R6,R5
        ADD     R6,R4           ;;BINARY NUMBER IN R6
        JR      LL008           ;;LOOP BACK FOR MORE DIGITS
LL009:  DEC     BUFPNTR         ;;BACK UP POINTER
LL010:  RCF                     ;;CLEAR CARRY
LL011:  LD      R4,R6           ;;NUMBER IS IN R4 NOW
        OR      R4,R4           ;;SET FLAGS
        RET
;;
;;              ** END OF COMMAND DECODER **
;;-----------------------------------------------------------------
;;HERE ARE THE MORSE CODE COMMANDS
;;
DIT:    LD      CT1200,#74              ;;62 MS DELAY  "." COMMAND
        JR      DIDAH
DAH:    LD      CT1200,#223             ;;186 MS DELAY  "-" COMMAND
DIDAH:  CALL    LSTXINIT                ;;SET UP LOW SPEED MODEM XMITTER
        OR      PFLAGS,#TXON            ;;TONE ON
        CALL    WAIT                    ;;WAIT FOR A WHILE
        AND     PFLAGS,#"TXON           ;;TONE OFF
        LD      CT1200,#74              ;;ANOTHER 62 MS DELAY
        JP      WAIT
;;
;;
;;------THIS IS THE "A" COMMAND-----------------------------
;;
A$COMM: CALL    W120                    ;;DELAY 120 MS
        AND     PFLAGS,#"MODE           ;;SET TO ANSWER MODE
        AND     IRQ,#"8                 ;;KILL PENDING KEYPRESS
;;
;; FALL INTO "GO$ONLINE" BELOW
;;
;;-------THIS IS THE "O" COMMAND-----------------------------
;;
GO$ONLINE:
        DI
        LD      SPL,#STACK
        LD      IMR,#20H
        CLR     IRQ4PTR
        CLR     IRQ3PTR
        CLR     RINGS
        EI
        CALL    SHON                    ;;GO OFF-HOOK
        TM      IMAGE34,#VCO            ;;SEE IF WE ALREADY ARE ONLINE
        JR      Z,GOL1                  ;;JUMP IF COMPLETE HANDSHAKE NOT REQ.
GOL0:   TM      PFLAGS,#MODE            ;;TEST FOR ORIG/ANSW MODE
        JP      Z,ANSWR                 ;;FULL ANSWER HANDSHAKE IF ZERO
        TCM     PFLAGS,#(OBAUD0+OBAUD1)
        JP      NZ,ORIG$LOW             ;;LOW SPEED ORIGINATE
        JP      ORIG$HIGH               ;;HIGH SPEED ORIGINATE
;;
GOL1:   
        TCM     PFLAGS,#(OBAUD0+OBAUD1) ;;CHECK BAUD RATE
        JR      Z,GOL2                  ;;JUMP IF HIGH SPEED
        TM      PFLAGS,#MODE            ;;CHECK ORIG/ANSW MODE
        JR      Z,GOL3                  ;;JUMP IF ANSWER
        CALL    LSRXIO                  ;;CALL LOW SPEED RECV. ORIG INIT
        JP      ORIG$LOW2               ;;THEN GO ON LINE
GOL3:   CALL    LSRXIA                  ;;CALL LOW SPEED RECV. ANSW INIT
        JP      ANSWR2                  ;;THEN GO ON LINE
GOL2:   LD      RESULTS,#'5'            ;;CONNECT 1200 RESULT
        TM      PFLAGS,#MODE            ;;CHECK MODE
        JR      Z,GOL4                  ;;JUMP IF HIGH SPEED ANSWER
        CALL    HSRXIO                  ;;CALL HIGH SPEED RECV ORIG INIT
        JP      AWAIT17                 ;;GO ON LINE
GOL4:   CALL    HSRXIA                  ;;CALL HIGH SPEED RECV ANSW INIT
GOL5:   JP      AWAIT17                 ;;GO ON LINE
;;
;;
;;-------SPEAKER CONTROL COMMANDS  M0,M1,M2 ------------------
;;
SPCRL:  CALL    NEXTNUM                 ;;GET THE 0,1,2
        RR      R4                      ;;GET BITS IN RIGHT PLACE
        RR      R4
        AND     OFLAGS,#"(SPON+SPON2)   ;;CLEAR THE DESTINATION
        AND     R4,#(SPON+SPON2)        ;;KEEP ONLY SPEAKER CONTROL BITS
        OR      OFLAGS,R4               ;;PUT THEM IN "OFLAGS"
        TM      IMAGE35,#SWHOOK         ;;SEE IF WE ARE OFF-HOOK
        JP      NZ,SHON                 ;;DO SPEAKER CONTROL IF OFF HOOK
        RET
;;
;;---------------THIS IS THE "=" COMMAND-----------------------
;;
SETRAM: CP      ADDRESS,#(RINGN+18)
        JP      UGE,GETCOMM             ;;ADDRESS TOO BIG, ERROR
        CP      ADDRESS,#RINGN
        JP      ULT,GETCOMM             ;;ADDRESS TOO SMALL, ERROR
;;                                      ;;FALL INTO "ANYLOC" BELOW IF OK
;;
;;---------------THIS IS THE ":" COMMAND------------------------
;;
ANYLOC: CALL    NEXTNUM                 ;;GET DATA;;
        LD      @ADDRESS,R4             ;;PUT IN RAM
        RET                             ;;DONE;;
;;
;;---------------THIS IS THE "S" COMMAND------------------------
;;
SETADR: CALL    NEXTNUM                 ;;GET ADDRESS OF S REGISTER
        ADD     R4,#RINGN               ;;ADD OFFSET TO ADDRESS
        LD      ADDRESS,R4
        RET
;;---------------THIS IS THE " ? " COMMAND------------------------
GETRAM:
GETANY:                                 ;;
        CALL    WAIT250                 ;;"?" COMMAND <-----
        JR      Z,BDEC                  ;;NO LEADING CRLF IF NON VERBOSE;;
        CALL    CRLF
BDEC:   LD      R4,@ADDRESS             ;;GET DATA BYTE
BDEC2:  LD      R7,#P10TAB&0FFH         ;;POINT TO POWER OF 10 TABLE;;
        LD      R6,#P10TAB>8
LL003:  LDC     R5,@RR6                 ;;GET A POWER OF TEN;;
        INC     R7                      ;;POINT TO NEXT POWER;;
        LD      TXHOLD,#'0'-1
LL004:  INC     TXHOLD                  ;;COUNT POWERS OF TEN;;
        SUB     R4,R5
        JR      NC,LL004
        CALL    CHAROUT                 ;;PRINT ASCII DIGIT;;
        ADD     R4,R5                   ;;RESTORE TO PLUS;;
        CP      R5,#1
        JR      NE,LL003                ;;LOOP FOR ALL POWERS;;
        JP      CRLF                  ;;DONE, DO END OF LINE STUFF;;
;;----------------------------------------------------------------
;;
SETECHO:
        LD      R7,#LEON                ;;"E0,E1" COMMANDS  <----
        LD      R8,#OFLAGS
        JR      SETFLAGS
;;
SETVERBOSE:
        LD      R7,#VBON                ;;"V0,V1" COMMANDS  <----
        LD      R8,#OFLAGS
        JR      SETFLAGS
;;
SETQUIET:
        LD      R7,#QUIET               ;;"Q0,Q1" COMMANDS <----
        LD      R8,#OFLAGS
        JR      SETFLAGS
 
SETFULL:
        LD      R7,#DPLX                ;;"F0,F1" COMMANDS <----
        LD      R8,#PFLAGS
;;
SETFLAGS:
        LD      R9,@R8                  ;;GET THE BYTE TO MODIFY
        OR      R9,R7                   ;;SET MASK FIRST
        CALL    NEXTNUM                 ;;GET THE 0 OR 1 FROM BUFFER
        JR      NZ,LL066                ;;IF NOT ZERO THEN QUIT NOW
        COM     R7                      ;;ELSE INVERT R7 MASK
        AND     R9,R7                   ;;AND CLEAR THE BITS
LL066:  LD      @R8,R9                  ;;PUT  THE BYTE BACK
        RET
;;
;;
;;
REVERSE:
        XOR     PFLAGS,#MODE            ;;"R" COMMAND  <----
        RET
;;
ENABPULSE:
        AND     OFLAGS,#"TDIAL          ;;"P" COMMAND  <----
        RET
;; 
ENABTONE:
        OR      OFLAGS,#TDIAL           ;;"T" COMMAND  <----
        RET
;;
;;-------------------------------------------------------------------
;;THIS IS THE PRODUCT ID CODE GENERATOR  (THE "I" COMMAND)
;;ALSO, "I1" WILL RETURN THE CHECKSUM OF THE PROM
;;
PRODUCT$ID:
        CALL    WAIT250                 ;;DELAY AND GET VERBOSE FLAG
        JR      Z,PID0                  ;;NO LEADING CRLF IF NON-VERBOSE
        CALL    CRLF                    ;;CRLF IF VERBOSE
PID0:   CALL    NEXTNUM                 ;;GET 0 OR 1, AND MAYBE SET ZERO FLAG
        LD      R4,#123                 ;;GET PRODUCT ID, 123  (REV 3)
        JR      Z,PID2                  ;;JUMP IF PRODUCT ID REQUEST
        LD      R6,#0FH                 ;;POINT TO TOP ROM LOCATION
        LD      R7,#0FFH                ;;  /
        CLR     R4                      ;;CLEAR CHECKSUM
PID1:   LDC     R5,@RR6                 ;;GET A ROM BYTE
        ADD     R4,R5                   ;;ADD IT TO CHECKSUM
        DECW    RR6                     ;;POINT TO NEXT ADDRESS
        JR      PL,PID1                 ;;LOOP TIL IT GOES TO FFFF
PID2:   JR      BDEC2                   ;;NOW GO PRINT IT TO TERMINAL
;;
;;--------------------------------------------------------------------
;;
;;THIS IS THE DIALING ROUTINE
 
DIAL:
        CALL    W120                    ;;WAIT 120 MS FOR POSSIBLE LINE FEED
        DI
        LD      SPL,#STACK
        LD      IMR,#20H                ;;ONLY MASTER TIMEBASE INTERRUPTS ENAB.
        EI
        OR      PFLAGS,#MODE            ;;SET FOR ORIGINATE MODE
        TM      IMAGE35,#SWHOOK         ;;SEE IF WE ALREADY ARE OFF HOOK
        JR      NZ,LL057                ;;  / SKIP SWHOOK ROUTINE IF OFF HOOK
        CALL    SHON                    ;;ELSE GO OFF HOOK AND WAIT 2 SECONDS
LL057:  AND     IRQ,#"8                 ;;KILL PENDING KEYPRESS FLAG
        LD      R4,DTW                  ;;GET DIAL TONE WAIT TIME
        SUB     R4,#2                   ;;SUBTRACT MIN. DIAL TONE WAIT
        JR      ULE,DLOOP               ;;NO EXTRA WAIT IF DTW <= 2 SECONDS
        CALL    WAITSEC                 ;;ELSE DO EXTRA TIME
DLOOP:
        CALL    NEXTCHR                 ;;NEXT CHAR. FROM BUFFER
        JP      C,GO$ONLINE             ;;GO WAIT FOR CARRIER IF END OF BUFFER
LL058:  LD      R5,#12                  ;;NUMBER OF TONES IN TABLE
        LD      R8,#TONETBL > 8         ;;HI TABLE POINTER
        LD      R9,#TONETBL & 255       ;;LOW TONE TABLE POINTER
LL059:  LDC     R6,@RR8                 ;;GET CHAR FROM TABLE
        CP      R4,R6                   ;;MATCH INPUT??
        JR      EQ,FOUND$DIGIT
        ADD     R9,#5                   ;;POINT TO NEXT ENTRY
        ADC     R8,#0
        DJNZ    R5,LL059                ;;AND LOOP BACK
        LD      R8,#CMDT2-3 > 8
        LD      R9,#CMDT2-3 & 255       ;;POINT TO COMMAND TABLE
LL060:  ADD     R9,#3                   ;;ADVANCE POINTER
        ADC     R8,#0
        LDC     R6,@RR8                 ;;LOOK FOR SPECIALS NOW
        CP      R6,#0                   ;;TEST FOR END OF TABLE (R6=0)
        JR      EQ,DLOOP                ;;YES, NO SUCH COMMAND
        CP      R4,R6                   ;;DOES IT MATCH COMAND IN R4?
        JR      NE,LL060                ;;IF NOT THEN GET NEXT COM. TO COMPARE
;;                                      ;;ELSE FALL INTO NEXT ROUTINE
FOUNDCOMMAND:
        INCW    RR8              ;;POINT TO COMMAND ADDRESS
        LDC     R6,@RR8         ;;GET HIGH BYTE TO R6
        INCW    RR8             ;;POINT TO LOW BYTE
        LDC     R7,@RR8         ;;GET LOW BYTE TO R7
        CALL    @RR6            ;;DO COMMAND @RR6
LL061:  JR      DLOOP           ;;BACK TO TOP OF LOOP FOR NEXT COMMAND
 
FOUND$DIGIT:
        DI
        LD      IRQ5VH,#DIALER$TB > 8   ;;POINT VECTORS TO DIALER TIMEBASE
        LD      IRQ5VL,#DIALER$TB & 255 ;; /
        OR      IMAGE35,#F1200HZ+CARRIER ;;SET TX FILTER TO 700-1600 RANGE
        CALL    UPDATE35                ;; / AND FORCE CARRIER DETECT FALSE
        CALL    ZERO$FREQ               ;;TURN OFF TONE GENERATOR
        EI                              ;;ENABLE INTERRUPTS
        INCW    RR8                     ;;POINT TO TONE FREQ.
        TM      OFLAGS,#TDIAL           ;;SEE IF WE DO TONE DIALING
        JR      Z,PULSEDIAL             ;;JUMP IF PULSE DIALING
        DI
        LD      R6,#PRIMREG+2           ;;POINTER TO PRIMARY REG 2
        LDCI    @R6,@RR8                ;;HIGH BYTE OF HIGH TONE TO PRIM. R2
        LDCI    @R6,@RR8                ;;LOW BYTE OF HIGH TONE TO PRIM R3
        LD      R6,#INTREG+10           ;;POINT TO INT. REG 10
        LDCI    @R6,@RR8                ;;HIGH BYTE OF LOW TONE TO INT. R10
        LDCI    @R6,@RR8                ;;LOW BYTE OF LOW TONE TO INT. R11
        EI                              ;;TONE STARTS NOW
        CALL    TLWAIT                  ;;WAIT FOR TONE DURATION
        CALL    ZERO$FREQ               ;;TURN OFF THE TONE
        CALL    TLWAIT                  ;;WAIT
DIGDONE:DI
        LD      IRQ5VH,#TIMEBASE > 8    ;;SET FOR MAIN TIME BASE ROUTINE
        LD      IRQ5VL,#TIMEBASE & 255  ;; /
        EI
        JP      DLOOP                   ;;BACK FOR NEXT DIGIT
;;
;;
TLWAIT: TM      LPBACK,#2               ;;CHECK FOR LONG TONE TEST MODE
        JR      NZ,TLWAIT               ;;WAIT TILL KEY PRESSED IF TEST MODE
        LD      R1,TONLEN               ;;ELSE GET LENGTH OF TOUCH TONE
LL062:  LD      R0,#10                  ;;PREPARE TO COUNT 1.04 MS INTERVALS
LL063:  OR      R0,R0                   ;;ELSE TEST R0 FOR ZERO COUNT
        JR      NZ,LL063
        DJNZ    R1,LL062                ;;COUNT 1.04 MS PERIODS
        RET
;;
;        
;;THIS IS THE PULSE DIALER...  R5 HAS NUMBER OF PULSES
;;
PULSEDIAL:
        CP      R5,#11                  ;;0..9 RANGE?
        JR      UGE,DIGDONE             ;;EXIT IF NOT VALID DIGIT
LL064:  DI
        AND     IMAGE35,#"SWHOOK
        CALL    UPDATE35                ;;GO ON-HOOK
        LD      R1,#59                  ;; 59*1.04167 = 61.4583 MS BREAK
        EI
        CALL    LL062                   ;;WAIT....
        DI
        OR      IMAGE35,#SWHOOK
        CALL    UPDATE35                ;;GO OFF-HOOK AGAIN
        LD      R1,#37                  ;;37*1.04167 = 38.5417 MS MAKE
        EI
        CALL    LL062                   ;;WAIT
        DJNZ    R5,LL064                ;;LOOP BACK FOR MORE CLICKS
        LD      R5,#5
LL065:  CALL    W120                    ;;INTERDIGIT DELAY = 600 MS
        DJNZ    R5,LL065
        JR      DIGDONE                 ;;FINISHED DIALING 1 DIGIT
;;
;;
ZERO$FREQ:
        AND     PFLAGS,#"TXON           ;;TRANSMITTER OFF
ZF0:    PUSH    IMR
        DI
        LD      R3,#6
        LD      R2,#INTREG+10
ZF1:    CLR     @R2                     ;;CLEAR INTREG 10-15
        INC     R2
        DJNZ    R3,ZF1
        CLR     R2
        LD      INTREG+2,#STATEF > 8    ;;SET INDIRECT POINTERS
        LD      INTREG+3,#STATEF & 255
        LD      INTREG+8,#ZERO > 8
        LD      INTREG+9,#ZERO & 255
        POP     IMR
        RET
;;
;;
;;
STOP$DIAL:                              ;;THIS IS THE ";" COMMAND <-----
        DI
        CALL    ZERO$FREQ
        LD      IRQ5VH,#TIMEBASE > 8    ;;SET FOR MAIN TIME BASE ROUTINE
        LD      IRQ5VL,#TIMEBASE & 255  ;; /
        EI
        JP      EXECUTE                 ;;EXIT WITHOUT GOING ONLINE BUT OFFHOOK
;;
;;
HOOKCRL:
        CALL    NEXTNUM                 ;; TEST FOR H0,H1,H2
        JR      Z,SHOFF                 ;;H0 = SWITCH HOOK OFF
        RCF
        DJNZ    R4,SHONLY               ;;IF H2 DONT PULL RJ13 RELAY
SHON:   SCF                             ;;SET CARRY FOR NORMAL
SHONLY: AND     IMAGE35,#"SPEAKER       ;;KILL SPEAKER FIRST
        TM      OFLAGS,#(SPON+SPON2)    ;;SEE IF HE WANTS SPEAKER ON
        JR      Z,NOSPKR                ;;JUMP IF NOT
        JR      MI,ALWAYS               ;;JUMP IF ON ALWAYS
        TM      IMAGE34,#40H            ;;SEE IF CD LED IS ON
        JR      NZ,NOSPKR               ;;NO SPEAKER IF CARRIER PRESENT
ALWAYS: OR      IMAGE35,#SPEAKER        ;;SET SPEAKER ON BIT
NOSPKR: TM      IMAGE35,#SWHOOK         ;;TEST SWITCH HOOK BIT
        JR      NZ,HRET                 ;;DONT DO IT AGAIN IF OFF HOOK
        JR      NC,NORJ13               ;;TEST THE CARRY (SET AT "SHON")
        OR      IMAGE35,#RJ13           ;;CLOSE RJ13 RELAY
NORJ13: OR      IMAGE35,#SWHOOK         ;;CLOSE SWITCH HOOK RELAY
        CALL    UPDATE35                ;;MOVE IMAGE35 TO EXTERNAL BITS
        LD      R4,#2                   ;;SET FOR 2 SECOND WAIT
        JP      WAITSEC
HRET:   JP      UPDATE35
;;
SHOFF:  CALL    CD$OFF                  ;;TURN OFF CARRIER DETECT LED
        OR      IMAGE34,#VCO+ANSW       ;;RECEIVER VCO OFF FOR BASEBAND 
        AND     IMAGE35,#"(SWHOOK+RJ13+SPEAKER)
        OR      IMAGE35,#(CARRIER+RING)
        AND     PFLAGS,#"TXON           ;;KILL MODEM TRANSMITTER
        JP      UD3435                  ;;UPDATE 34,35
;;
;;
DIALER$TB:
        SRP     INTREG
        LD      R0,#SINE256 > 8 ;;POINT TO SINE TABLE
        LD      R1,R12
        OR      R1,#80H
        LDC     R2,@RR0         ;;GET THE SINE OF VECTOR 1 INTO R2
        LD      R1,R14          ;;
        OR      R1,#80H
        LDC     R0,@RR0         ;;GET SINE OF VECTOR 2 INTO R0
        ADD     R0,R2           ;;SUM THE VECTORS
        ADD     R0,R0           ;;MPY BY 4
        ADD     R0,R0
        LD      DAC,R0          ;;OUT TO THE DAC
        CLR     R2              ;;R2 MUST BE ZERO SO NOT TO SCREW UP
;;                              ;;/  THE MAIN TIME BASE ROUTINE!
        ADD     R13,PRIMREG+3   ;;ADVANCE PHASOR 1
        ADC     R12,PRIMREG+2
        ADD     R15,R11         ;;ADVANCE PHASOR 2
        ADC     R14,R10
        SRP     PRIMREG         ;;BACK TO PRIMARY REGISTERS
        DEC     R0              ;;DEC R0 AS A PRESCALER
        TM      R0,#7           ;;MOD 8
        JR      NZ,TB$DONE      ;;JUMP IF LS 3 BITS NOT ALL ZERO
        LD      PORT0,#DTR      ;;ADDRESS RS232 DTR LEAD
        TM      IRQ,#8          ;;TEST FOR KEY PRESSED
        JR      NZ,ABORT$DIAL   ;;ABORT IF KEY PRESSED
        TM      PORT3,#8        ;;TEST FOR FALSE DTR
        JR      NZ,ABORT$DIAL   ;;ABORT IF DTR FALSE
        OR      CT1200,CT1200   ;;NOW DEAL WITH 1200 HZ COUNTER/TIMER
        JR      Z,TB$DONE
        DEC     CT1200          ;;COUNT DOWN 1200 HZ COUNTER
TB$DONE:IRET
;;
;;
ABORT$DIAL:
        LD      RESULTS,#'0'            ;;OK RESULT CODE
ABORT:  DI                              ;;NO INTERRUPTS NOW
        LD      IRQ5VH,#TIMEBASE > 8    ;;SET VECTOR TO MAIN HS TIMEBASE
        LD      IRQ5VL,#TIMEBASE & 255  ;;  /
        CALL    ZERO$FREQ               ;;KILL TRANSMITTER
        CALL    SHOFF                   ;;HANG UP PHONE
        JP      GETCOMM
;;
;;
;;THIS IS THE DTMF FREQUENCY DECODING TABLE FOR THE DIAL ROUTINE;;
;;
TONETBL:
        DB    '#'
        DW    F1477
        DW    F941
        DB    '*'
        DW    F1209
        DW    F941
        DB    '0'
        DW    F1336
        DW    F941
        DB    '9'
        DW    F1477
        DW    F852
        DB    '8'
        DW    F1336
        DW    F852
        DB    '7'
        DW    F1209
        DW    F852
        DB    '6'
        DW    F1477
        DW    F770
        DB    '5'
        DW    F1336
        DW    F770
        DB    '4'
        DW    F1209
        DW    F770
        DB    '3'
        DW    F1477
        DW    F697
        DB    '2'
        DW    F1336
        DW    F697
        DB    '1'
        DW    F1209
        DW    F697

;;---------------------------------------------------------------
;;
;;THIS PUTS THE MODEM IN ANSWER MODE FOR INCOMING CALLS
;;
ANSWR:
        TCM     PFLAGS,#(OBAUD0+OBAUD1) ;;TEST FOR HIGH SPEED
        JR      NZ,ANSWR1               ;;JUMP IF LOW
        TM      LPBACK,#1               ;;TEST FOR LOOP BACK MODE
        JR      NZ,ANSW$LPB             ;;JUMP IF HIGH SPEED LOOP BACK
        CP      LCDT,#255               ;;IGNORE CARRIER DETECT?
        JR      EQ,ANSW12               ;;/ YES, GO TO HIGH SPEED ANSWER NOW
ANSWR1: CALL    LSTXINIT                ;;SET UP TX FOR LOW SPEED OPERATION
        OR      PFLAGS,TXFLAG           ;;TURN ON TRANSMITTER
        LD      BITCOUNT,CDW            ;;GET CARRIER DETECT WAIT TIME
ANSWR3: CALL    LSRXIA                  ;;SET UP LOW SPEED RECEIVER
        CALL    WAIT$FSK$CD             ;;WAIT FOR A SIGNAL
        JP      NC,NO$CARRIER           ;;ABORT IF NO CARRIER
        LD      R0,#40                  ;;SET FOR 200 MS MARK QUALIFICATION
        CALL    FSKM                    ;;SEE IF ITS A MARK FOR 200 MS
        JR      NC,ANSW12               ;;JUMP IF NOT A MARK (POSSIBLE 1200)
ANSWR2: JR      ORIG$LOW2
;;
ANSW$LPB:
        CALL    HSTXINIT                ;;IF LOOPBACK TURN ON XMITTER NOW
        OR      PFLAGS,TXFLAG           ;;TURN ON TRANSMITTER
        LD      BITCOUNT,CDW            ;;GET CARRIER WAIT TIME
ANSW12: CALL    HSRXIA                  ;;INITIALIZE HIGH SPEED RECEIVER
        SUB     BITCOUNT,#2             ;;COUNT EXTRA 2 SECONDS
        JP      ULE,NO$CARRIER          ;;GIVE UP AFTER TIMEOUT (ZERO OR CARRY)
        PUSH    BITCOUNT                ;;SAVE SECONDS TIMEOUT COUNTER
        LD      BITCOUNT,#2             ;;SET TO LOOK FOR PSK MARK FOR 2 SEC.
        CALL    WAIT$PSK$CD             ;;WAIT FOR HIGH SPEED CARRIER
        POP     BITCOUNT
        JP      NC,ANSWR3               ;;IF NO CARRIER TRY LOW SPEED AGAIN
        CALL    HSTXINIT                ;;SET TRANSMITTER FOR HIGH SPEED
        OR      PFLAGS,TXFLAG           ;;  / AND TURN ON TRANSMITTER
        CALL    WAIT250                 ;;WAIT 1/2 SECOND
        CALL    WAIT250                 ;;**BUG FIX**
        JR      AWAIT17                 ;;THEN GO ON-LINE
;;
        
;;------------------------------------------------------------
;;
;;THIS PUTS THE MODEM IN LOW SPEED ORIGINATE MODE FOR OUT-GOING CALLS
;;
ORIG$LOW:
        CALL    LSTXINIT                ;;INITIALIZE LOW SPEED TRANSMITTER
        CALL    LSRXIO                  ;;INIT. LOW SPEED RECEIVER
        LD      BITCOUNT,CDW            ;;GET CARRIER DETECT WAIT TIME
        CALL    WAIT$FSK$CD             ;;WAIT FOR LOW SPEED CARRIER
        JP      NC,NO$CARRIER
        CALL    FSK$MARK
        JP      NC,NO$CARRIER
        OR      PFLAGS,TXFLAG   
ORIG$LOW2:
        OR      IMAGE35,#HS$LED         ;;TURN OFF HS LED
        LD      RESULTS,#'1'            ;;LOW SPEED CONNECT RESULT CODE
        JR      LSENTRY                 ;;NOW GO DO LOW SPEED MODEM ONLINE
;;
;;---THIS PUTS THE MODEM IN HIGH SPEED ORIGINATE MODE--------   
;;
ORIG$HIGH:
        LD      BITCOUNT,CDW            ;;GET CARRIER DETECT WAIT TIME
        CALL    HSTXINIT                ;;INITIALIZE HIGH SPEED TRANSMITTER
        TM      LPBACK,#1               ;;TEST FOR LOOP BACK MODE
        JR      NZ,OH001                ;;JUMP IF LOOP BACK
        CALL    LSRXIO                  ;;INIT. LOW SPEED  RECEIVER
        CALL    WAIT$FSK$CD             ;;WAIT FOR LOW SPEED CARRIER
        JP      NC,NO$CARRIER
        CALL    FSK$MARK
        JP      NC,NO$CARRIER
OH001:  CALL    HSRXIO                  ;;INITIALIZE HIGH SPEED RECEIVER
        OR      PFLAGS,TXFLAG           ;;TURN HIGH SPEED TRANSMITTER ON
        CALL    WAIT$PSK$CD             ;;WAIT FOR HIGH SPEED CARRIER
        JP      NC,NO$CARRIER
AWAIT17:AND     IMAGE35,#"HS$LED        ;;TURN ON HIGH SPEED LED
        LD      RESULTS,#'5'            ;;CONNECT 1200 RESULT
        DI
        LD      IMR,#20H                ;;TURN OFF ALL INT. EXCEPT TRANSMIT
        EI
LSENTRY:CALL    RESULT                  ;;SEND RESULT CODE
        CALL    CD$ON                   ;;TURN ON CARRIER DETECT LED
        CALL    SHON                    ;;UPDATE SPEAKER
        TM      IMAGE34,#PSK            ;;TEST FOR HIGH SPEED MODE
        JR      Z,CLOOPB                ;;JUMP IF LOW SPEED
        DI
        LD      R7,#0B0H                ;;MUTE RECEIVER
        LD      IMR,#31H                ;;TURN ON RECEIVER AGAIN
        AND     IRQ,#20H                ;;KILL PENDING INTERRUPTS
        AND     IMAGE34,#"TCA           ;;KILL TX CHAR AVALABLE FLAG
        EI
        LD      CT1200,#60              ;; 50 MS DELAY SO DESCRAMBLER CAN
CLOOPA: CP      CT1200,#0               ;;  / GET BACK IN SYNC.
        JR      NZ,CLOOPA
        LD      R7,#0B4H                ;;UNMUTE RECEIVER
CLOOPB: TM      IMAGE34,#PSK            ;;SEE IF HIGH OR LOW SPEED MODE
        JR      Z,CL00A                 ;;JUMP IF LOW SPEED
        OR      PFLAGS,#(OBAUD0+OBAUD1+IBAUD0+IBAUD1)  ;;SET FOR FAST RESULTS
        JR      CL00B
CL00A:  TCM     PFLAGS,#(OBAUD0+OBAUD1) ;;HIGH SPEED LAST TIME?
        JR      NZ,CL00B                ;;NO, LEAVE BAUD BITS ALONE
        AND     PFLAGS,#"(OBAUD0+IBAUD0);;YES, NOW SET FOR 300 BAUD RESULTS
        OR      PFLAGS,#(OBAUD1+IBAUD1)
CL00B:  CLR     R1                      ;;CLEAR ESCAPE TIMER
        CLR     BITCOUNT                ;;CLEAR ESCAPE CHARACTER COUNTER
        OR      PFLAGS,#ONLINE          ;;SET ON-LINE STATUS
CLOOP:
        CP      CT1200,#0               ;;SERVICE GUARD TIME COUNTERS
        JR      NE,CLOOP1               ;;  /
        LD      CT1200,#24              ;;20 MS TIMEOUT
        CALL    DTRCHECK                ;;TEST DTR AND ABORT IF FALSE   
        CALL    CD$UPDATE               ;;MAKE CD LED AGREE WITH CARRIER STATUS
        CP      LCDT,#255               ;;IGNORE CARRIER DETECT?
        JR      EQ,CL000
        TM      IMAGE34,#CARRIER        ;;SEE IF CARRIER IS PRESENT
        JR      Z,CD$RECOVER            ;; / (UPDATED BY CD$UPDATE)
CL000:  OR      R1,R1                   ;;TEST GUARD TIME COUNTER FOR ZERO
        JR      Z,CLOOP1                ;; /
        DEC     R1                      ;;DECREMENT IF IT WASNT ZERO
CLOOP1: TM      IMAGE34,#TCA            ;;TEST FOR TRANSMITTER CHAR AVALABLE
        JR      Z,CL005                 ;;JUMP IF ZERO (NO CHARACTER)
        AND     IMAGE34,#"TCA           ;;CLEAR THE TX CHAR FLAG
        TCM     PFLAGS,#(OBAUD0+OBAUD1) ;;TEST FOR HIGH SPEED
        JR      NZ,CL001                ;;JUMP IF LOW SPEED
        TM      PFLAGS,#DPLX            ;;TEST FOR HALF DUPLEX MODE
        JR      NZ,CL001                ;;JUMP IF FULL DUPLEX (NO ECHO)
        LD      R10,INTREG+11           ;;PUT TRANSMITTED CHAR IN RX HOLDING
        OR      R7,#40H                 ;;SET DATA RECEIVED FLAG TRUE
CL001:  AND     INTREG+11,#7FH          ;;CLEAR PARITY BIT ON TX CHAR
        TM      OFLAGS,#STUPID          ;;ESCAPE SEQUENCE ALLOWED?
        JR      NZ,CLOOP                ;; / JUMP IF ESCAPE NOT ALLOWED
;;
;;ESCAPE CODE DETECTOR
;;
        CP      BITCOUNT,#0             ;;IS THIS THE FIRST ESCAPE CHARACTER?
        JR      NZ,CL002                ;;JUMP IF NOT FIRST CH AFTER GUARD TIME
        OR      R1,R1                   ;;TEST FOR EXPIRED GUARD TIME
        JR      NZ,CL003                ;;/ JUMP IF NOT EXPIRED
CL002:  CP      INTREG+11,ESCODE        ;;IS THIS AN ESCAPE CHARACTER?
        JR      NE,CL003                ;;/ JUMP IF NOT
        INC     BITCOUNT                ;;COUNT THIS ESCAPE CHARACTER
        JR      CL004                   ;;
;;
CL003:  CLR     BITCOUNT                ;;CLEAR ESCAPE CHARACTER COUNTER
CL004:  LD      R1,EGT                  ;;RELOAD GUARD TIME COUNTER
CL005:  OR      R1,R1                   ;;IS GUARD TIME EXPIRED?
        JR      NZ,CLOOP                ;; / JUMP IF NOT
        CP      BITCOUNT,#3             ;;3 ESCAPE CHARS YET?
        JR      EQ,ESC$DET              ;; / JUMP IF 3 HAVE BEEN RECEIVED
        CP      EGT,#0                  ;;IS GUARD TIME SET TO ZERO?
        JR      Z,CLOOP                 ;;/ YES, LOOP BACK TO TOP
        CLR     BITCOUNT                ;;ELSE CLEAR ESCAPE CHARACTER COUNTER
        JR      CLOOP                   ;; / THEN GO BACK TO TOP
ESC$DET: LD     R7,#0B0H                ;;MUTE RECEIVER
        JP      OK                      ;;GOTO COMMAND PROCESSOR
;;---------
CD$RECOVER:                             ;;COME HERE WHEN CARRIER IS LOST
        LD      R2,#255                 ;; **BUG FIX** \/  \/
        LD      R1,#255
RECOV1: INC     R1                      ;;ADVANCE LOST CARRIER DISCONN. TIMER
        INC     R2                      ;; / AND CARRIER DELAY TIMER
        LD      CT1200,#120             ;;SET FOR 100 MS DELAY
RECOV2: CP      CT1200,#0               ;;100 MS UP?
        JR      EQ,RECOV1               ;;YES, ADVANCE TIMERS
        TM      IMAGE35,#CARRIER        ;;/NO, TEST FOR PRESENCE OF CARRIER
        JR      Z,RECOV3                ;;JUMP IF CARRIER PRESENT
        CLR     R2                      ;;/  ELSE JUST RESET CARRIER TIMER
        JR      RECOV4                  ;;
RECOV3: CP      R2,CDELAY               ;;CARRIER PRESENT LONG ENOUGH?
        JP      UGE,CL00B               ;; / YES, GO BACK ON LINE
RECOV4: CP      R1,LCDT                 ;;CARRIER GONE LONG ENOUGH?
        JP      EQ,NO$CARRIER           ;; / YES, EXIT
        JR      RECOV2                  ;;/  ELSE WAIT SOME MORE
;;
;;---------
CD$UPDATE:
        TM      IMAGE35,#CARRIER        ;;CHECK STATE OF HARDWARE CARRIER DET.
        JR      Z,CD$ON1                ;; / JUMP IF IT IS ON
        TM      IMAGE34,#40H
        JR      Z,CD$RET
CD$OFF: PUSH    IMR
        DI
        LD      R7,#0B0H                ;;   MUTE RECEIVER      
        LD      PORT3,R7                ;;   CLAMP RS232 DATA TO MARK
        LD      R0,#0E0H                ;; / SET TO TURN OFF CD LED
        AND     IMAGE34,#"40H           ;;/  SAVE STATE OF CD-LED
        JR      SETRI2                  ;;
;;
CD$ON1: TM      IMAGE34,#40H            ;;SEE IF LED WAS ON ALREADY
        JR      NZ,CD$RET               ;;DONT BOTHER IF LED ALREADY ON
CD$ON:  PUSH    IMR
        DI
        OR      IMAGE34,#40H            ;;SAVE NEW STATE OF CD-LED
        LD      R0,#60H                 ;;CD LED ON
        LD      R7,#0B4H                ;;UN-MUTE RECEIVER
SETRI2: LD      PORT0,R0                ;;ADDRESS THE BIT IN THE LATCH
        NOP                             ;;DELAY
        AND     PORT3,#"20H             ;;STROBE THE ADDRESSABLE LATCH
        OR      PORT3,#20H
        POP     IMR
CD$RET: RET

;;-------------------------------------------------------------
;;INITIALIZE THE HIGH SPEED RECEIVER HARDWARE AND INTERRUPT DRIVERS.
;;
HSRXIO: LD      IMAGE34,#(PSK+LOFREQ)   ;;ORIGINATE MODE INIT.
        JR      HSRXI
HSRXIA: LD      IMAGE34,#(PSK+ANSW)     ;;ANSWER MODE INIT.
HSRXI:  CALL    UPDATE34                ;;SET HARDWARE
        DI
        LD      IRQ4PTR,#2              ;;POINT IRQ4 TO 1219 HZ INT. ROUTINE
        LD      R7,#0B0H                ;;INIT. RECEIVER FLAGS
        LD      PRE0,#29                ;;DIVIDE BY N=7
        LD      T0,#117                 ;;DIV. BY 117 FOR 1219 HZ
        OR      TMR,#3                  ;;START TIMER 0 RUNNING
        LD      IMR,#31H                ;;ENABLE RECEIVER INTERRUPTS
        EI
        RET
;;
;;------------------------------------------------------------------
;;INITIALIZE THE LOW SPEED RECEIVER HARDWARE AND INTERRUPT DRIVERS
;;
LSRXIO: LD      IMAGE34,#0              ;;LOW SPEED ORIGINATE
        JR      LSRXI
LSRXIA: LD      IMAGE34,#ANSW           ;;LOW SPEED ANSWER
LSRXI:  CALL    UPDATE34                ;;SET HARDWARE
        DI
        LD      IMR,#20H                ;;ONLY MAIN TIMEBASE INT. ACTIVE
        LD      R7,#0B0H                ;;INIT RECEIVER FLAGS
        EI
        RET
;;
;;---------------------------------------------------------------
;;THIS SETS THE HIGH SPEED MODEM TRANSMITTER TO THE CORRECT
;;FREQUENCY BASED ON THE MODE AND LOOPBACK CONTROL BITS.
;;IT DOES NOT TURN ON THE TRANSMITTER.
;;** R0 IS MODIFIED
HSTXINIT:
        DI
        LD      IRQ5VH,#TIMEBASE > 8    ;;SET VECTOR TO HS TRANSMITTER
        LD      IRQ5VL,#TIMEBASE & 255  ;; /
        CALL    ZF0                     ;;CLEAR TRANSMITTER VARIABLES
        TM      PFLAGS,#MODE            ;;TEST FOR ORIG. / ANSW. MODES
        JR      Z,HSANSW                ;;JUMP IF ANSWER
        TM      LPBACK,#1               ;;TEST FOR LOOP BACK MODE
        JR      NZ,HSANSW1              ;;ANSWER FREQ IF LOOPBACK
HSORIG: LD      INTREG+10,#F1200 > 8    ;;ORIGINATE CARRIER = 1200 HZ
        LD      INTREG+11,#F1200 & 255  ;; /
        OR      IMAGE35,#F1200HZ        ;;TUNE FILTER HARDWARE TO 1200 HZ
        JR      HSTX1
HSANSW: TM      LPBACK,#1               ;;TEST FOR ANSWER MODE LOOPBACK
        JR      NZ,HSORIG               ;;JUMP IF LOOP BACK
HSANSW1:LD      INTREG+10,#F2400 > 8    ;;ANSWER CARRIER = 2400 HZ
        LD      INTREG+11,#F2400 & 255  ;; /
        AND     IMAGE35,#"F1200HZ       ;;TUNE FILTER HARDWARE TO 2400 HZ
HSTX1:  CALL    UPDATE35                ;;SET TRANSMIT FILTER HARDWARE
        LD      COUNTER,#1              ;;INIT ASYNC. TO SYNC CONVERTER
        LD      TXDATA,#255             ;; /
        EI                              ;; /
        RET
;;
;;----------------------------------------------------------------
;;INITIALIZE THE LOW SPEED TRANSMITTER
;;NOTE: IT IS NOT TURNED ON WITH THIS ROUTINE
;;
LSTXINIT:
        DI
        LD      IRQ5VH,#LOW$TIMEBASE >8 ;;POINT JUMP VECTOR TO LOW SPEED 
        LD      IRQ5VL,#LOW$TIMEBASE & 255
        SRP     INTREG
        TM      PFLAGS,#MODE            ;;TEST FOR ORIG OR ANSW
        JR      NZ,LS0                  ;;JUMP IF LOW SPEED ORIGINATE
        TM      LPBACK,#1
        JR      NZ,LS$ORIG              ;;REVERSE IF LOOP BACK
LS$ANSW:LD      R6,#F2025 > 8
        LD      R7,#F2025 & 255
        LD      R8,#F2225 > 8
        LD      R9,#F2225 & 255
        AND     IMAGE35,#"F1200HZ       ;;TUNE TX FILTER TO 2400 HZ
        JR      LSDONE
LS0:    TM      LPBACK,#1               ;;TEST FOR LOOP BACK MODE
        JR      NZ,LS$ANSW              ;;REVERSE MODES IF TRUE
LS$ORIG:LD      R6,#F1070 > 8
        LD      R7,#F1070 & 255
        LD      R8,#F1270 > 8
        LD      R9,#F1270 & 255
        OR      IMAGE35,#F1200HZ        ;;TUNE TX FILTER TO 1200 HZ
LSDONE: CLR     R12
        CLR     R13                     ;;CLEAR CARRIER REGISTERS
        LD      R11,#255
        CLR     COUNTER
        LD      R15,#4                  ;;ASSUME 300 BAUD
        TM      PFLAGS,#OBAUD1
        JR      NZ,LS1
        LD      R15,#11                 ;;OR SET FOR 110 BAUD
LS1:    LD      R14,R15
        LD      R5,#1
        SRP     PRIMREG                 ;;RESTORE PRIMARY REGISTER SET
        CALL    UPDATE35                ;;SET FILTER HARDWARE
        EI
        RET
;;
;;-----------------------------------------------------------------
;;HIGH SPEED MODE RECEIVER CARRIER DETECT.
;;RETURNS CARRY SET IF CARRIER DETECTED
;;
WAIT$PSK$CD:
        CP      LCDT,#255               ;;IGNORE CARRIER DETECT?
        JR      EQ,AWC11A               ;; / YES, RETURN AS IF CARRIER PRESENT
        CLR     R3                      ;;CLEAR MARK COUNTER
AWC9:   LD      R0,#10                  ;;COUNT 100 MS INTERVALS
AWC10:  LD      CT1200,#120             ;;SET FOR 100 MS TIMEOUT
AWC11:  CP      R3,#120            ;;120 MARK BITS? (COUNTED BY "DATACLK")
        JP      ULE,AWC12       ;;(WAS 240) ;;120 MARKS=100 MILLISECONDS DELAY
AWC11A: SCF                             ;;SET CARRY FLAG
        AND     IMAGE35,#"HS$LED        ;;ENABLE HS LED
AWC11B: RET
;;
AWC12:  CALL    KP$ABORT                ;;ABORT IF KEY PRESSED
        CP      CT1200,#0               ;;TEST FOR 100 MS TIMEOUT
        JR      NE,AWC11
        CALL    DTRCHECK                ;;CHECK ON FALSE DTR
        DJNZ    R0,AWC10                ;;JUMP IF 1 SECOND NOT UP
        DEC     BITCOUNT                ;;COUNT DOWN MAX SECONDS
        JR      NZ,AWC9
AWC13:  RCF
        RET                             ;;RETURN CARRY CLEAR IF TIME UP
;;-------------------------------------------------------------------
;;WAITS FOR STEADY MARK CARRIER TONE AND RETURNS CARRY SET IF CARRIER PRESENT.
;;RETURNS CARRY CLEAR IF: KEY PRESSED, DTR FALSE,
;; OR NO CARRIER WITHIN CARRIER DETECT WAIT TIME SET BY S7  "CDW".
;;MODIFIES R0,R1,R2,R3,R4,BITCOUNT,CT1200,PORT0
;;
WAIT$FSK$CD:
        EI
        CP      LCDT,#255               ;;IGNORE CARRIER DETECT?
        JR      EQ,WFC4B                ;;/ EXIT AS IF CARRIER PRESENT IF EQ
        CLR     R0                      ;;R0, R1 ARE CARRIER PRESENT TIMERS
        CLR     R1
WFC1:   LD      R2,#100                 ;;R2 COUNTS SECONDS
WFC2:   LD      CT1200,#12              ;;10 MS DELAY
WFC3:   CP      CT1200,#0
        JR      NE,WFC3 
        TM      IMAGE35,#CARRIER        ;;TEST FOR CARRIER ENERGY
        JR      NZ,WFC5                 ;;JUMP IF NO CARRIER PRESENT
        TM      PFLAGS,#MODE            ;;TEST FOR ORIG/ANSW
        JR      Z,WFC4                  ;;DONT CARE ABOUT MARK IF ANSWER MODE
        DI
        LD      PORT0,#RXFSK
        NOP
        TM      PORT3,#8                ;;TEST FOR MARK OR SPACE
        EI
        JR      Z,WFC5                  ;;JUMP IF NOT MARK
WFC4:   INC     R1
        CP      R1,#10                  ;;100 MS OF CARRIER YET?
        JR      NE,WFC4A                ;;JUMP IF NOT
        INC     R0
        CLR     R1                      ;;ELSE COUNT 1/10 SECONDS
WFC4A:  CP      R0,CDELAY               ;;CARRIER DELAY EXPIRED?
        JR      NE,WFC6                 ;;JUMP IF CARRIER NOT PRESENT
WFC4B:  SCF                             ;;SET CARRY
        RET                             ;;AND RETURN WITH CARRIER PRESENT
;;
WFC5:   CLR     R0
        CLR     R1
WFC6:   CALL    KP$ABORT                ;;CHECK FOR KEY PRESSD
        DJNZ    R2,WFC2                 ;;COUNT 1/10 SECOND
        CALL    DTRCHECK                ;;CHECK FOR FALSE DTR
        DEC     BITCOUNT                ;;COUNT SECONDS         
        JR      NZ,WFC1
WFC7:   RCF                             ;;RESET CARRY FLAG
        RET                             ;;RETURN WITH NO CARRIER
;;
DTRCHECK:
        DI
        LD      PORT0,#DTR              ;;CHECK ON RS232 DTR PIN
        NOP
        TM      PORT3,#8
        EI
        JR      KPA0
;;
KP$ABORT:
        TM      OFLAGS,#STUPID          ;;DUMB MODE?
        JR      NZ,KPA1                 ;;DONT LOOK AT KEYBOARD IF DUMB
        TM      IRQ,#8                  ;;KEY PRESSED?
KPA0:   JR      NZ,NO$CARRIER           ;;ABORT IF TRUE
KPA1:   RET
;;
NO$CARRIER:
        LD      RESULTS,#'3'    ;;NO CARRIER RESULT
        JP      ABORT           ;;EXIT TO COMMAND LEVEL VIA ABORT ROUTINE
;;
;;
;;---------------------------------------------------
;;LOOKS FOR AN FSK MARK TONE FOR 100 MS.
;;IF MARK IS PRESENT FOR THE ENTIRE PERIOD IT RETURNS CARRY SET.
;;IF SPACE IS DETECTED IT RETURNS WITH CARRY CLEAR.
;;DISTURBS  R0,R1,PORT0,CT1200
;;
FSK$MARK:
        LD      R0,#20          ;;FOR 100 MS TIMEOUT
FSKM:   EI
        CP      LCDT,#255       ;;IGNORE CARRIER?
        JR      EQ,FSKM2A       ;;YES, RETURNS AS IF CARRIER PRESENT
FSKM0:  LD      CT1200,#6       ;;SET FOR 5 MS TIMEOUT
FSKM1:  CP      CT1200,#0
        JR      NZ,FSKM1
        DI
        LD      PORT0,#RXFSK    ;;ADDRESS FSK DATA PORT
        NOP
        LD      R1,PORT3        ;;GET MARK OR SPACE
        EI
        TM      PFLAGS,#MODE    ;;TEST FOR ORIG/ANSW
        JR      NZ,FSKM2        ;;JUMP IF ORIG
        COM     R1
FSKM2:  TM      R1,#8           ;;TEST FOR MARK
        JR      Z,FSKM3         ;;JUMP IF NOT A MARK
        DJNZ    R0,FSKM0        ;;LOOP 10 TIMES
FSKM2A: SCF                     ;;SET CARRY
        RET                     ;;MARK PRESENT
FSKM3:  RCF                     ;;ELSE CLEAR CARRY
        RET                     ;;MARK NOT PRESENT
;;
;;--------THIS IS THE C0,C1 COMMAND--------------------------
;;
SETCARRIER:
        CALL    NEXTNUM
        JR      NZ,CXON
        AND     PFLAGS,#"TXON           ;;CARRIER OFF
        CLR     TXFLAG                  ;;NEVER TO COME ON AGAIN!
        RET
;;
CXON:   DI              
        TCM     PFLAGS,#(OBAUD0+OBAUD1)  ;;TEST FOR HIGH/LOW SPEED
        JR      Z,CXON1                 ;; / JUMP IF HIGH SPEED
        CALL    LSTXINIT                ;;SET UP LOW SPEED XMITTER
        JR      CXON2
CXON1:  CALL    HSTXINIT                ;;SET UP HIGH SPEED TRANSMITTER
CXON2:  LD      TXFLAG,#TXON            ;;SET TRANSMITTER CONTROL FLAG
        OR      PFLAGS,TXFLAG           ;;TURN ON TRANSMITTER
        RET             ;;NOTE: INTERRUPTS ENABLED BY "LSTXINIT" OR "HSTXINIT".
;;
;;---------------------------------------------------------------
;;THESE ARE THE TIME DELAY ROUTINES
;;
;;
WAIT2SEC:
        LD      R4,PT           ;;GET PAUSE TIME FOR COMMA
WAITSEC:AND     R4,R4           ;;TEST R4 FOR ZERO
        JR      Z,WAIT$DONE     ;;EXIT WHEN R4 IS ZERO
        CALL    WAIT250
        CALL    WAIT250
        CALL    WAIT250
        CALL    WAIT250         ;;WAIT 1 SECOND
        DEC     R4
        JR      WAITSEC
;;
WAIT250:
        LD      CT1200,#150     ;;WAITS 250 MS
        CALL    WAIT
W120:   LD      CT1200,#150     ;;WAITS 125 MS
WAIT:   OR      CT1200,CT1200   ;;TEST FOR ZERO
        JR      NZ,WAIT         ;;LOOP TIL CT1200 IS ZERO
WAIT$DONE:
        TM      OFLAGS,#VBON    ;;RETURN STATE OF VERBOSE FLAG
        RET


;;-------------------------------------------------------
;;RECEIVER DIBIT DECODER
;;
;;RR14 USED AS DIBIT AND DATA BIT BUFFER ( GLOBAL!)
;;CONTENTS OF RR12 DESTROYED
;;
;;REGISTERS USED:
;;      R15     SCRATCH
;;      R14     BIT 7 HOLDS DESCRAMBLED DATA BIT
;;      R13     SCRATCH
;;      R12     SCRATCH
;;      R11     LINE RATE SHIFT REGISTER
;;      R10     RECEIVER HOLDING REGISTER
;;      R9      OUTPUT SHIFT REGISTER
;;      R8      TIME DELAY COUNTER
;;      R7      STATUS FLAGS
;;      R6      OUTPUT BIT COUNTER
;;      R5      INPUT BIT COUNTER
;;      R4      COUNTS CONTINUOUS ZERO BITS (FOR BREAK DETECTION)
;;      R3      COUNTS 1 BITS
;;R7 BIT MAP:
;;      BIT 2   1=OK TO SEND RX DATA TO RS232    0= RX DATA MUTED
;;      BIT 3   BREAK DETECTED
;;      BIT 4   ALWAYS 1
;;      BIT 5   ALWAYS 1
;;      BIT 6   CHARACTER IN RECEIVER HOLDING REGISTER IS READY
;;      BIT 7   NEXT BIT TO BE OUTPUT TO RS232
;;      
DIBIT:
        LD      PORT0,#DIBIT1
        AND     IMR,#"4         ;;KILL THIS INTERRUPT (IRQ2)
        LD      R12,PORT3       ;;GET DIBIT 1 INTO R12
        LD      PORT0,#DIBIT2   
        AND     IRQ,#"1         ;;KILL PENDING IRQ0
        OR      IMR,#1          ;;ENABLE IRQ0
        LD      R13,PORT3       ;;GET DIBIT 2 TO R13
        EI                      ;;ENABLE INTERRUPTS
DIB0:   TM      PFLAGS,#MODE    ;;CHECK FOR ORIG/ANSW
        JR      NZ,DIB1         ;;JUMP IF ORIG MODE
        COM     R12             ;;ELSE COMPENSATE FOR INVERTED RX SPECTRUM
DIB1:   SWAP    R12             ;;GET DIBITS TO MSB POSITION
        SWAP    R13
        ADD     R12,R12         ;;SHIFT DIBIT 2 INTO CARRY
        ADC     R15,R15         ;;THEN INTO DIBIT REGISTER
        ADD     R13,R13         ;;SHIFT DIBIT 1 INTO CARRY
        ADC     R15,R15         ;; THEN INTO DIBIT REGISTER
        AND     R15,#0FH        ;;KEEP LAST FOUR DIBITS
        CLR     R12
        LD      R13,#DBDECODE & 255 ;;FORM POINTER INTO DECODING TABLE
        ADD     R13,R15
        ADC     R12,#DBDECODE > 8
        LDC     R14,@RR12       ;;GET VALUE FROM TABLE TO R14
        JR      DESCRAMBLE
;;
;;
DATACLK:
        AND     IMR,#"1         ;;TURN OFF THIS INTERRUPT (IRQ0)
        AND     IRQ,#"4         ;;KILL PENDING IRQ2
        OR      IMR,#4          ;;ENABLE IRQ2
        EI
DESCRAMBLE:                     ;;LS BIT OF R14 HAS THE DATA
        RR      DSH             ;;PUT BIT 17 INTO CARRY
        TM      DSM,#20H        ;;TEST BIT 14
        JR      Z,DES1
        CCF                     ;;EXOR BIT 17 WITH BIT 14
DES1:   TM      R14,#1          ;;TEST DATA BIT
        JR      Z,DES2          ;;JUMP IF ZERO
        CCF                     ;;EXOR WITH DATA
DES2:   RRC     R14             ;;SHIFT CARRY INTO MSB OF R14
        ADC     DSL,DSL
        ADC     DSM,DSM
        ADC     DSH,DSH         ;;SHIFT THE SCRAMBLER REGISTER
DES3:   INC     R3              ;;ASSUME MARK
        OR      R14,R14         ;;TEST MSB OF R14
        JR      MI,DB10         ;;JUMP IF IT REALLY WAS A MARK
        CLR     R3              ;;ELSE CLEAR THE MARK COUNTER
;;
DB10:   TM      R8,#255         ;;TEST DELAY TIMER
        JR      MI,DB11         ;;JUMP IF MINUS
        DEC     R8              ;;ELSE COUNT DOWN
DB11:   JR      NZ,DB12         ;;JUMP IF NOT ZERO YET
        OR      R7,#40H         ;;IF ZERO SET THEN SET READY FLAG
DB12:   OR      R5,R5           ;;TEST R5 (INPUT BIT COUNTER)
        JR      NZ,DB14         ;;JUMP IF IN A CHARACTER
        TM      R14,#80H        ;;TEST FOR START BIT
        JR      NZ,DB15         ;;JUMP IF NO START BIT, THEN CLEAR BREAK FLAG
        LD      R5,#1           ;;START BIT FOUND, SET BIT COUNTER=1
        LD      R4,#1           ;;AND SET SPACE COUNTER=1
DB13:   IRET
;;
DB14:   CP      R5,#9           ;;SEE IF WE ARE IN THE STOP BIT
        JR      UGE,INSTOP
        INC     R5              ;;COUNT ANOTHER BIT
        LD      FLAGS,R14       ;;PUT RECEIVED DATA BIT INTO CARRY FLAG
        RRC     R11             ;;THEN SHIFT CARRY INTO LINE SHIFT REGISTER
        JR      MI,DB15         ;;JUMP IF BIT WAS A 1
        INC     R4              ;;ELSE ADVANCE SPACE COUNTER IF ZERO
        JR      DB16
DB15:   CLR     R4              ;;CLEAR SPACE COUNTER IF BIT WAS 1
        AND     R7,#"8          ;;AND CLEAR BREAK FLAG
DB16:   CP      R4,#22          ;;TEST FOR 22 SPACE BITS (BREAK)
        JR      LT,DB17         ;;JUMP IF NOT BREAK
        OR      R7,#8           ;;ELSE SET BREAK FLAG
DB17:   IRET                    ;;AND EXIT
;;
INSTOP: CLR     R5              ;;RESET INPUT BIT COUNTER
        TM      R14,#80H        ;;TEST FOR MISSING STOP BIT
        JR      NZ,GOTSTOP      ;;JUMP IF ITS THERE
        INC     R5              ;;ELSE COUNT THE START BIT
GOTSTOP:LD      R10,R11         ;;PUT DATA BYTE INTO HOLDING REGISTER
        LD      R8,#4           ;;4 BIT DELAY TILL READY
        IRET                    ;;EXIT
;;
;;THE FOLLOWING IS THE CODE WHICH OUTPUTS THE RECEIVED DATA
;;TO THE RS232 PORT AT 1219 BAUD.  IT IS ENTERED ONCE FOR
;;EACH 1219 HZ INTERRUPT OF COUNTER/TIMER (T0)
;;
OUT1219:
        TM      R7,#4           ;;TEST THE MUTE CONTROL BIT
        JR      Z,ODB0          ;;EXIT IF MUTED
        LD      PORT3,R7        ;;OUTPUT DATA BIT TO RS232
ODB0:   EI                      ;;ENABLE INTERRUPTS
        OR      R6,R6           ;;TEST OUTPUT BIT COUNTER FOR ZERO
        JR      NZ,ODB1         ;;JUMP IF IN A CHARACTER (NON-ZERO)
        TM      R7,#40H         ;;SEE IF INPUT CHARACTER IS READY IN HOLDING
        JR      Z,ODB2          ;;JUMP IF NOT AND EXIT
        LD      R9,R10          ;;GET HOLDING REGISTER DATA INTO SHIFT REGISTER
        AND     R7,#"0C0H       ;;START BIT=0  READY FLAG=0
        LD      R6,#1           ;;LOAD BIT COUNTER=1
ODB2:   IRET                    ;;EXIT
;;
ODB1:   CP      R6,#9           ;;STOP BIT NEXT?
        JR      UGE,DOSTOP      ;;YES, JUMP AND PROCESS IT IF R6 > 8
        INC     R6              ;;NO, JUST PROCESS NEXT BIT
        RR      R9              ;;SHIFT OVER NEXT BIT TO OUTPUT
        JR      C,ODB3
        AND     R7,#"80H        ;;OUTPUT ZERO
        IRET
ODB3:   OR      R7,#80H         ;;OUTPUT ONE
        IRET
;;
DOSTOP: TM      R7,#8           ;;IS THIS A BREAK
        JR      NZ,YESBREAK
        OR      R7,#80H         ;;IF NOT BREAK THEN DO A STOP BIT
        CLR     R6              ;;AND CLEAR BIT COUNTER
        IRET                    ;;EXIT
;;
YESBREAK:
        AND     R7,#"80H        ;;DONT DO STOP BIT IF IN A BREAK
        IRET                    ;;EXIT
;;      
;;DIBIT DECODING TABLE USED BY "DIBIT:"
;;
DBDECODE:               ;;FOR 212A
        DB      2,0,3,1
        DB      3,2,1,0
        DB      0,1,2,3
        DB      1,3,0,2
;;END OF TABLE
;;
;;--------------------------------------------------------------
;;RESULT CODE TRANSMITTING ROUTINE
;; R0,R1,R2 AND CT1200 ARE USED.
;;
RESULT:
        CP      RESULTS,#'5'            ;;CONNECT 1200 RESULT?
        JR      NE,RSL1
        TM      USTAT,#EXR              ;;EXTENDED RESULT MODE?
        JR      NZ,RSL1                 ;;YES, JUMP
        LD      RESULTS,#'1'            ;;NO, CHANGE TO NORMAL CONNECT  
RSL1:   TM      OFLAGS,#QUIET           ;;QUIET MODE?
        JP      NZ,RTN
        CALL    WAIT250                 ;;WAIT 250 MS & TEST VERBOSE FLAG
        JR      Z,NONVB                 ;;JUMP IF NOT VERBOSE
PRNMSG: CALL    CRLF
        LD      R0,#OKMSG>8             ;;BASE OF MESSAGES
        LD      R1,#OKMSG&255
        LD      R2,#'0'                 ;;START WITH ASCII 0
LL005:  PUSH    R0
        LDC     R0,@RR0                 ;;GET A MESSAGE CHAR.
        LD      TXHOLD,R0
        POP     R0
        CP      TXHOLD,#'$'             ;;DELIMITER?
        JR      NE,LL006                ;;JUMP IF NOT
        INC     R2                      ;;ELSE ADVANCE COUNTER
        JR      LL007
LL006:  CP      R2,RESULTS              ;;PAST OUR MESSAGE YET?
        JP      UGT,CRLF                ;;YES, FINISH WITH CRLF
        JR      NE,LL007                ;;NOT THERE YET, JUMP
        CALL    CHAROUT                 ;;PRINT IT
LL007:  INCW    RR0                     ;;POINT TO NEXT CHARACTER
        JR      LL005
;;
;;
NONVB:  LD      TXHOLD,RESULTS          ;;GET RAW RESULT CODE
        CALL    CHAROUT                 ;;SEND IT
;;
CRLF:   LD      TXHOLD,CRC              ;;<CR>
        TM      OFLAGS,#VBON            ;;TEST FOR VERBOSE
        JR      Z,CHAROUT               ;;JUST <CR> IF NON-VERBOSE
        CALL    CHAROUT
        LD      TXHOLD,LFC              ;;<LF>
;;
;;FALL INTO CHAROUT BELOW....
;;
;; UART TRANSMITTER ROUTINE
;; ENTER WITH DATA TO XMIT IN "TXHOLD"
;;  USES R13,R14,R15,CT1200,BITCOUNT
CHAROUT:
        LD      BITCOUNT,#254
        LD      CT1200,#255
        SCF                     ;;SET CARRY FOR "1" BIT
        CALL    BITOUT          ;;SEND LEADING STOP BIT
        SCF
        CALL    BITOUT          ;;AGAIN FOR A TOTAL OF 2
        CLR     R14             ;;CLEAR PARITY
        TM      USTAT,#EPE      ;;TEST FOR EVEN PARITY
        JR      NZ,LL040        ;;JUMP IF EVEN SELECTED
        INC     R14             ;;ELSE INIT. FOR ODD PARITY
LL040:  LD      R15,#7          ;;ASSUME 7 DATA BITS
        TM      USTAT,#DBITS    ;;CHECK FOR 8 SELECTED
        JR      Z,LL041         ;;JUMP IF 7
        INC     R15             ;;ELSE SET FOR 8
LL041:  LD      R13,USTAT       ;;GET USTAT TO R0
        AND     R13,#DB8        ;;KEEP THE STATE OF DATA BIT 8
        OR      R13,TXHOLD      ;;OR IN THE DATA TO SEND
        RCF                     ;;CLEAR CARRY FOR A ZERO
        CALL    BITOUT          ;; <--- SEND THE START BIT
LL042:  RRC     R13             ;;SHIFT A DATA BIT INTO CARRY FLAG
        JR      NC,LL043        ;;DONT MESS WITH PARITY IF ZERO
        INC     R14
LL043:  CALL    BITOUT          ;; <--- SEND A DATA BIT
        DJNZ    R15,LL042       ;;LOOP TILL ALL DATA BITS SENT
        TM      USTAT,#PE       ;;PARITY ENABLED ??
        JR      Z,LL044         ;;JUMP IF PARITY NOT ENABLED
        RRC     R14             ;;PUT PARITY IN CARRY FLAG
        CALL    BITOUT          ;; <--- SEND PARITY BIT
LL044:  LD      PORT3,#0F0H     ;; NOW SEND THE STOP BIT
RTN:    RET
;;
;;                              OBAUD1  OBAUD0
;;                                0       0     INVALID
;; OUTPUTS 1 BIT....              0       1     110 BAUD
;;                                1       0     300 BAUD
;;                                1       1     1200 BAUD
BITOUT: JR      C,LL045
        LD      PORT3,#70H      ;;OUTPUT ZERO BIT
        JR      LL046
LL045:  LD      PORT3,#0F0H     ;;OUTPUT ONE BIT
;;
;;NOW DELAY 1 BIT TIME...
;;
LL046:  TCM     PFLAGS,#OBAUD0+OBAUD1  ;;TEST FOR 1200 BAUD
        JR      NZ,LL047A
        SUB     BITCOUNT,#1             ;;DO 1200 BAUD
        JR      LL047
LL047A: TM      PFLAGS,#OBAUD1  ;;TEST FOR 300 BAUD
        JR      Z,LL047B
        SUB     BITCOUNT,#4             ;;DO 300 BAUD
        JR      LL047
LL047B: SUB     BITCOUNT,#11            ;;DO 110 BAUD
LL047:  CP      CT1200,BITCOUNT ;;COMPARE COUNTER WITH REFERENCE
        JR      NC,LL047        ;;WAIT TILL COUNTER < R3
        RET
;;
;;---------------------------------------------------------
;;VERBOSE RESULT CODES....
;;
OKMSG:  DB      'OK$'
CDMSG:  DB      'CONNECT$'
RIMSG:  DB      'RING$'
LCMSG:  DB      'NO CARRIER$'
ERMSG:  DB      'ERROR$'
HSCONN: DB      'CONNECT 1200$'
;;
;;----------------------------------------------------------
;;
;;THIS IS THE UART RECEIVER
;;REGISTERS USED: R5,R6,R7
;;
IRQ4:                           ;;COME HERE FOR TIMER 0 INTERRUPTS
        CP      IRQ4PTR,#1      ;;DATA BIT COLLECTION IF =1
        JP      EQ,DATABITS
        JP      GT,OUT1219      ;;IF =2 THEN DO 1200 BPS RECEIVER STUFF
OVERFLOW:
        DEC     BITCOUNT        ;;COUNT OVERFLOWS
        JR      NZ,OVRDONE      ;;JUMP IF NOT TOO MANY
        CLR     IRQ3PTR         ;;RESET BACK TO BEGINNING
        AND     IMR,#"10H       ;;KILL T0 INTERRUPTS
        AND     TMR,#"3         ;;AND STOP T0
OVRDONE: IRET
;;
;;
FINDBAUD:
        AND     TMR,#0FCH       ;;STOP T0
        LD      T0,#255         ;;LOAD T0
        LD      PRE0,#1         ;;DIV. BY 64... 64.1026 US PER COUNT
        LD      IRQ3PTR,#1      ;;CHANGE IRQ3 POINTER TO "COMPUTEBAUD"
        CLR     IRQ4PTR         ;;POINT T0 INTERRUPT TO OVERFLOW HANDLER
        LD      BITCOUNT,#2     ;;USED TO COUNT OVERFLOWS OF T0
        DI
        AND     IRQ,#"18H       ;;KILL PENDING T0 AND RS232 DATA INTERRUPT
        OR      TMR,#3          ;;START T0 RUNNING
        OR      IMR,#18H        ;;ENABLE T0 AND RS232 DATA INTERRUPTS
        IRET
;;
;;
COMPUTEBAUD:                    ;;COME HERE ON NEXT 1->0 TRANSISTION IN "A"
        AND     TMR,#"03        ;;STOP T0
        LD      IRQ3PTR,#2      ;;IRQ3 NOW POINTS TO "STARTBIT"
        LD      R14,#80H        ;;ASSUME LSB WAS 1 FOR THE "A"
        OR      PFLAGS,#IBAUD0+IBAUD1  ;;SET THE BAUD SELECT BITS  (11)
        CP      BITCOUNT,#1     ;;TEST FOR OVERFLOWS (110 BAUD OR LESS)
        LD      BITCOUNT,#2     ;;PRESET BIT COUNTER TO 2 ONLY FOR THE "A"
        JR      GT,CBD0         ;;JUMP IF GREATER THAN 110 BAUD
        XOR     PFLAGS,#IBAUD1  ;;SET BAUD BITS FOR 110  (01)
        JR      CBD1            ;;THEN EXIT
CBD0:
        CP      T0,#213         ;;COMPARE WITH 750 BAUD VALUE
        JR      UGT,CBD1        ;;JUMP IF 1200 BAUD
        XOR     PFLAGS,#IBAUD0  ;;NOW ASSUME 300 BAUD       (10)
        JR      CBD1
;;
;;
;;
IRQ3:   AND     IMR,#"8         ;;KILL RS232 DATA INTERRUPTS
        EI                      
        CP      IRQ3PTR,#1      ;;1= FIRST 1 TO 0 TRANSITION IN THE "A"
        JR      EQ,COMPUTEBAUD
        JR      LT,FINDBAUD     ;;0= "A" START BIT INTERRUPT
;;
STARTBIT:                       ;;COME HERE WHEN IRQ3PTR=2
        CLR     BITCOUNT
CBD1:   AND     TMR,#"3         ;;STOP T0
        LD      PRE0,#(32*4)+1  ;;SET FOR DIVIDE BY N= 32 CONT.
        LD      T0,#13          ;;SET FOR 1/2 BIT TIME AT 1200 BAUD
        TCM     PFLAGS,#(IBAUD0+IBAUD1)         ;;TEST FOR 1200 BAUD
        JR      Z,CBD2          ;;JUMP IF 1200
        LD      T0,#52          ;;SET FOR 1/2 BIT TIME AT 300 BAUD
        TM      PFLAGS,#IBAUD0  ;;CHECK FOR 300
        JR      Z,CBD2          ;;JUMP IF 300
        LD      T0,#142         ;;ELSE SET FOR 110
CBD2:   OR      TMR,#3          ;;START THE BIT TIMER
        LD      IRQ4PTR,#1      ;;SET IRQ4 (T0) TO DATA BIT ROUTINE
        OR      LPBACK,#80H     ;;SET 1/2 BIT TIME FLAG
        DI
        AND     IRQ,#"10H       ;;KILL PENDING T0 INTERRUPT
        LD      IMR,#30H        ;;ENABLE T0 INTERRUPT (IRQ4), KILL STARTBIT INT
CBD3:   IRET
;;
DATABITS:                       ;;COME HERE EVERY TIME T0 TIMES OUT
        TM      USTAT,#IBF      ;;BUFFER FULL?
        JP      NZ,LL039        ;; / QUIT PROCESSING IF BUFFER IS FULL
        XOR     LPBACK,#80H     ;;FLIP THE 1/2 BIT TIME FLAG
        JR      MI,LL027        ;;PROCESS ONLY IF IN BIT CENTER
        LD      R15,PORT3       ;;GET RS232 DATA BIT INTO R15, BIT 0
        EI                      ;;ALLOW NESTED INTRRUPTS
        CP      BITCOUNT,#9     ;;IN PARITY OR STOP?
        JR      UGE,LL031       ;;YES, JUMP
        CP      BITCOUNT,#0     ;;SEE IF WE ARE STILL IN THE START BIT
        JR      EQ,LL028        ;;IF SO THEN JUMP
        RRC     R15             ;;PUT IT DATA BIT IN CARRY FLAG
        RRC     R14             ;;SHIFT IT INTO RX SHIFT REGISTER
LL026:  INC     BITCOUNT
LL027:  IRET
;;
LL028:  TM      R15,#1          ;;TEST FOR SPACE INPUT
        JR      Z,LL026         ;;JUMP IF VALID START BIT
LL029:  DI
        LD      IMR,#28H        ;;KILL T0 INT, ENABLE STARTBIT
        AND     IRQ,#"18H       ;;KILL PENDING IRQ3 & 4
LLDONE: IRET
;;
LL031:  DI
        AND     IRQ,#"8         ;;KILL PENDING STARTBIT INTERRUPT
        LD      PORT0,#DTR      ;;SELECT RS-232 DTR LEAD
        LD      IMR,#28H        ;;DISABLE T0, ENABLE STARTBIT INTERRUPTS
        TM      PORT3,#8        ;;TEST DTR LEAD
        EI
        JR      NZ,LL033        ;;JUMP IF DTR FALSE TO IGNORE COMMANDS
        TM      OFLAGS,#STUPID  ;;SEE IF WE TAKE COMMANDS
        JR      NZ,LL033        ;;JUMP IF STUPID BIT IS SET
        RRC     R15             ;;PUT RS232 DATA BIT INTO CARRY
        RLC     R5              ;;..THEN INTO LSB OF R5 (BITS 8&9 GO HERE)
        LD      RXHOLD,R14      ;;THIS IS THE COMPLETE RECEIVED CHAR.
        OR      USTAT,#RRF
        INC     BITCOUNT
        LD      R6,#'A'
        INC     CHARS
        JR      MI,LL032        ;;JUMP IF 1ST CHAR "A"
        JR      NZ,LL034        ;;JUMP IF PAST THE "T"
PARITY: LD      R6,CHAR1        ;;GET THE "A"
        ADD     R6,R6           ;;SHIFT MSB INTO CARRY
        RLC     R5              ;;THEN INTO R5, BIT 0
        LD      R6,RXHOLD       ;;GET THE "T"
        ADD     R6,R6           ;;SHIFT MSB INTO CARRY
        RLC     R5              ;;THEN INTO R5, BIT 0
        AND     R5,#0FH         ;;KEEP LOW 4 BITS
        LD      R7,#FORMAT&255  ;;MAKE POINTER TO FORMAT TABLE
        LD      R6,#FORMAT>8
        ADD     R7,R5           ;;INDEX INTO TABLE
        LDC     R5,@RR6         ;;GET CHAR. FORMAT CODE
        AND     USTAT,#"(PE+EPE+DBITS+DB8)
        OR      USTAT,R5        ;;PUT CODE INTO "USTAT"
;;
        LD      R6,PFLAGS               ;;GET BAUD RATE FLAGS
        SWAP    R6                      ;;MOVE THEM OVER TO OUTPUT SIDE
        AND     R6,#(OBAUD0+OBAUD1)     ;;KEEP ONLY OUTPUT BAUD CONTROL BITS
        AND     PFLAGS,#"(OBAUD0+OBAUD1);;CLEAR OLD OUTPUT BAUD BITS
        OR      PFLAGS,R6               ;;TRANSFER INPUT BAUD TO OUTPUT BAUD
;;
        LD      R6,#'T'
LL032:  LD      CHAR1,RXHOLD    ;;SAVE THIS CHAR
        AND     RXHOLD,#7FH     ;;STRIP PARITY BIT
        CP      RXHOLD,R6       ;;IS CHAR CORRECT? ( A OR T)
        JR      EQ,LLDONE2      ;;YES, DONE WITH THIS ONE
        CP      RXHOLD,#'/'     ;;REPEAT CHARACTER?
        JR      EQ,REPEATIT
LL033:  DI
        LD      CHARS,#-2       ;;FAULTY "A" ,"T" OR "/"
        CLR     IRQ3PTR         ;; / ABORT BACK TO "FINDBAUD" AGAIN
        JR      LL029           ;;/  TO LOOK FOR AN "A"
;;
LL034:  AND     RXHOLD,#7FH     ;;STRIP PARITY
        CP      RXHOLD,CRC      ;;CARRIAGE RETURN?
        JR      EQ,LL038        ;;
        CP      CHARS,#MAXCHARS ;;BUFFER FULL?
        JR      ULE,LL036       ;;JUMP IF NOT FULL
        OR      USTAT,#OVL      ;;ELSE SET OVERFLOW FLAG
LL035:  DEC     CHARS           ;;DISCARD THE CHARACTER
        IRET                    ;;DONE....
;;
LL036:  CP      RXHOLD,#' '     ;;SPACE OR CONTROL CHAR?
        JR      ULE,LL035       ;;DUMP SPACE OR CONTROL CHAR
        CP      RXHOLD,#7FH     ;;RUBOUT?
        JR      EQ,LL035        ;;DUMP IT TOO
        CP      RXHOLD,#60H     ;;LOWER CASE?
        JR      ULT,LL037
        SUB     RXHOLD,#20H     ;;CONVERT TO UPPER CASE
LL037:  LD      R6,CHARS        ;;GET CHARACTER COUNT
        ADD     R6,#BUFFER-1    ;;INDEX INTO BUFFER
        LD      @R6,RXHOLD      ;;PUT CHARACTER IN BUFFER
        IRET                    ;;FINISHED PROCESSING 1 CHARACTER
;;
LL038:  DEC     CHARS                   ;;ADJUST CHARACTER COUNT
LL039:  OR      USTAT,#IBF              ;;SET INPUT BUFFER FULL FLAG
        DI                              ;;DISABLE INTERRUPTS
        LD      IMR,#20H                ;;KILL ALL UART INTERRUPTS
LLDONE2:IRET                            ;;**FINISHED PROCESSING COMMAND LINE 
;;
REPEATIT:
        LD      CHARS,LCHARS            ;; <--- "A/" COMMAND EXECUTED HERE
        JR      LL039
;;      
;; **  END OF UART RECEIVER ROUTINE
;;
;;-------------------------------------------------------------
;;
NO$CHAR:                ;;COME HERE WHEN NO TX CHAR AVALABLE
        CLR     COUNTER
        TM      IRQ,#8          ;;CHECK FOR START BIT
        JR      Z,SINEWAVE      ;;EXIT IF NO START BIT
        TM      PFLAGS,#ONLINE  ;;ARE WE ONLINE?
        JR      Z,SINEWAVE      ;;/IF NOT THEN DONT DO CHARACTER STUFF
        LD      COUNTER,#76     ;;LOAD THE COUNTER
        JR      SINEWAVE
;;
TEST$STOP:              ;;COME HERE DURING STOP BIT
        LD      R11,R4          ;;TRANSFER TO DATA TO HOLDING REGISTER, R11
        OR      IMAGE34,#TCA    ;;SET TX CHARACTER AVALABLE FLAG
        LD      TXDATA,PORT3    ;;SAMPLE THE STOP BIT
        AND     IRQ,#"8         ;;CLEAR START BIT FLAG
        TM      TXDATA,#1       ;;CHECK FOR STOP BIT PRESENT
        JR      NZ,SINEWAVE     ;;EXIT IF A STOP BIT IS PRESENT
        LD      COUNTER,#80H    ;;THIS FLAGS A BREAK
        LD      TXDATA,#240     ;;15 BITS TIMES 2
        JR      SINEWAVE
;;
IN$BREAK:               ;;COME HERE DURING A BREAK
        CP      TXDATA,#0       ;;BREAK EXTENDER TIMED OUT?
        JR      Z,IB2           ;;JUMP IF SO
        SUB     TXDATA,#2       ;;ELSE COUNT DOWN BY 2 (LEAVE LSB ALONE)
        JR      SINEWAVE        ;;THEN EXIT
IB2:    TM      PORT3,#1        ;;STILL DOING A BREAK?
        JR      Z,SINEWAVE      ;;EXIT IF STILL DOING BREAK
        COM     TXDATA          ;;FORCE MARK ON TXDATA
        CLR     COUNTER         ;;CLEAR THE COUNTER
        AND     IRQ,#"8         ;;RESET START BIT FLAG
        JR      SINEWAVE        ;;EXIT
;;
;;
;;
TIMEBASE:
        SRP     INTREG
;;
;;ASYNC TO SYNC CONVERTER
;;
        DEC     COUNTER         ;;DEC. SAMPLE COUNTER
        JR      MI,NO$CHAR      ;;NOTHING DOING IF MINUS
        JR      Z,TEST$STOP     ;;IN STOP BIT IF ZERO
        JR      OV,IN$BREAK     ;;DOING A BREAK IF OVERFLOW
        TM      COUNTER,#7      ;;ASYNC SAMPLE IF LOW 3 BITS ZERO
        JR      NZ,SINEWAVE
        LD      TXDATA,PORT3    ;;SAMPLE TX DATA NOW
        LD      R0,TXDATA
        RRC     R0              ;;SHIFT INTO TX DATA BIT INTO CARRY
        RRC     R4              ;;THEN INTO BIT 7 OF R4
;;
SINEWAVE:
        TM      PFLAGS,#TXON    ;;SEE IF TRANSMITTER IS ON
        JR      NZ,A2S
        JP      @RR8            ;;DO NOTHING IF TX IS OFF
A2S:    LD      R0,#SINE256 > 8 ;;POINT TO SINE TABLE
        LD      R1,R12
        OR      R1,#80H
        LDC     R2,@RR0         ;;GET THE SINE OF VECTOR 1 INTO R2
;;
        LD      R1,R14          ;;
        OR      R1,#80H
        LDC     R0,@RR0         ;;GET SINE OF VECTOR 2 INTO R0
        ADD     R0,R2           ;;SUM THE VECTORS
        LD      DAC,R0          ;;OUT TO THE DAC
;;
        ADD     R12,R10
        ADD     R14,R10
        CLR     R2              ;;CLEAR HIGH BYTE OF STATE POINTER
        JP      @RR8            ;;DO MODULATION ROUTINE NOW
;;
;;
;;END OF THE 1200 BAUD TIMEBASE ROUTINE
;;
;;------------------------------------------------------------------
;;
;;***** THIS IS THE LOW SPEED FSK TIMEBASE AND MODEM TRANSMITTER ROUTINE **
;;***** ALSO, THE COMPLETE FSK RECEIVE LOGIC IS HERE.
;;
LOW$TIMEBASE:
        SRP     INTREG
        TM      PFLAGS,#TXON    ;;TRANSMITTER ON ?
        JR      Z,LTB           ;;BYPASS CARRIER GENERATOR IF OFF
        LD      R0,#SINE256 > 8
        LD      R1,R12
        OR      R1,#80H
        LDC     R0,@RR0         ;;GET SINE VALUE FROM TABLE
        ADD     R0,R0           ;;MPY BY 2
        LD      DAC,R0          ;;OUT TO REAL WORLD
;;
LTB:    TM      PFLAGS,#ONLINE
        JR      Z,FSKMARK
        LD      R0,PORT3        ;;GET THE TRANSMIT DATA FROM RS232 LINE
        RR      R0              ;;PUT THE BIT INTO CARRY AND BIT 7
        JR      C,FSKMARK
        ADD     R13,R7          ;;DO SPACE FREQUENCY
        ADC     R12,R6
        JR      FSKDONE
FSKMARK:ADD     R13,R9          ;;ELSE DO MARK FREQUENCY
        ADC     R12,R8
FSKDONE:
        DJNZ    R5,LTB0         ;;DIVIDE 9600 HZ DOWN TO 1200 HZ
        LD      R5,#8
        CP      CT1200,#0
        JR      Z,LTB0
        DEC     CT1200          ;;MAINTAIN 1200 HZ COUNTER
LTB0:   DJNZ    R14,NO$SAMPLE   ;;THIS IS THE BAUD COUNTER (R14)
        LD      R14,R15         ;;RELOAD BAUD COUNTER WITH 4 OR 11
LTB1:   DEC     COUNTER         ;;DECREMENTS AT 8 TIMES BIT RATE
        JR      MI,NO$CHR       ;;JUMP IF NO CHARACTER AVALABLE
        JR      Z,IN$STOP       ;;JUMP IF IN STOP BIT
        TM      COUNTER,#7      ;;SEE IF LOW 3 BITS ARE ZERO
        JR      NZ,NO$SAMPLE    ;;DONT SAMPLE BIT IF BITS NOT ALL ZERO
        RL      R0              ;;PUT BIT INTO CARRY
        RRC     R4              ;;THEN INTO SERIAL TO PARALLEL REGISTER
        SRP     PRIMREG
        IRET
;;
NO$SAMPLE:
        TM      PFLAGS,#DPLX    ;;TEST FOR HALF DUPLEX
        JR      Z,HALFDPX       ;;JUMP IF HALF DUPLEX
        LD      R0,#255         ;;IF FULL MAKE RECV THINK TX BIT IS MARK
HALFDPX: SRP    PRIMREG         ;;BACK TO PRIMARY REGISTERS NOW
        TM      PFLAGS,#ONLINE
        JR      Z,NO$SAM1       ;;IF OFF-LINE DONT MESS WITH R7
        LD      PORT0,#RXFSK    ;;ADDRESS FSK RECEIVER DATA
        OR      R7,#80H         ;;ASSUME MARK
        TM      R7,#4           ;;TEST FOR MUTED RECEIVER
        JR      Z,NO$SAM2       ;;PUT OUT MARK IF MUTED
        TM      PORT3,#8        ;;TEST RECEIVED DATA BIT
        JR      NZ,NO$SAM0
        AND     R7,#"80H        ;;ELSE RECEIVE A SPACE
NO$SAM0:TM      PFLAGS,#MODE    ;;TEST ORIG/ANSW MODE
        JR      NZ,NO$SAM2
        XOR     R7,#80H         ;;INVERT DATA IF ANSWER MODE
NO$SAM2:OR      INTREG+0,#7FH   ;;FORCE UN-NEEDED BITS TO ALL ONES **BUG FIX**
        AND     R7,INTREG+0     ;;MIX IN THE TRANSMITTED DATA (BIT 7) FOR ECHO
        LD      PORT3,R7        ;;OUTPUT TO RS232 IF NOT MUTED
NO$SAM1:IRET
;;
NO$CHR:
        LD      PORT0,#CARDET    ;;ADDRESS CARRIER DETECT STATUS
        OR      IMAGE35,#CARRIER ;;ASSUME NO CARRIER
        TM      PORT3,#8        ;;TEST FOR CARRIER DETECT
        JR      Z,NO$CHR1       ;;JUMP IF NO CARIER
        AND     IMAGE35,#"CARRIER  ;;ELSE SET CARRIER DETECT FLAG
;;
NO$CHR1:CLR     COUNTER
        TM      IRQ,#8          ;;CHECK FOR START BIT
        JR      Z,NO$SAMPLE
        LD      COUNTER,#76     ;;LOAD SAMPLING CONTROL COUNTER
        SRP     PRIMREG
        IRET
;;
IN$STOP:
        LD      R11,R4          ;;PUT CHAR IN HOLDING REGISTER
        OR      IMAGE34,#TCA    ;;SET TRANSMIT CHARACTER AVALABLE FLAG
        AND     IRQ,#"8         ;;KILL PENDING STARTBIT FLAG
        SRP     PRIMREG
        IRET    
;;
;;
;;
        ORG     0D80H
;;
;; SINE TABLE MUST START ON PAGE BOUNDRY.
SINE256:
        DB       16 , 16 , 17 , 18 , 19 , 19 , 20 , 21 
        DB       22 , 22 , 23 , 24 , 24 , 25 , 26 , 26 
        DB       27 , 27 , 28 , 28 , 29 , 29 , 30 , 30 
        DB       30 , 31 , 31 , 31 , 31 , 31 , 31 , 31 
        DB       31 , 31 , 31 , 31 , 31 , 31 , 31 , 31 
        DB       30 , 30 , 30 , 29 , 29 , 28 , 28 , 27 
        DB       27 , 26 , 26 , 25 , 24 , 24 , 23 , 22 
        DB       22 , 21 , 20 , 19 , 19 , 18 , 17 , 16 
        DB       15 , 15 , 14 , 13 , 12 , 12 , 11 , 10 
        DB       9 , 9 , 8 , 7 , 7 , 6 , 5 , 5 
        DB       4 , 4 , 3 , 3 , 2 , 2 , 1 , 1 
        DB       1 , 0 , 0 , 0 , 0 , 0 , 0 , 0 
        DB       0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 
        DB       1 , 1 , 1 , 2 , 2 , 3 , 3 , 4 
        DB       4 , 5 , 5 , 6 , 7 , 7 , 8 , 9 
        DB       9 , 10 , 11 , 12 , 12 , 13 , 14 , 15 
;;
;; PLUS 90 DEGREE PHASE SHIFT ROUTINES
;;
PL90S0:
        LD      R9,#PL90S1 & 255
        ADD     R15,# 1635  & 255
        ADC     R14,# 1635  > 8
        ADD     R13,#-1223  & 255
        ADC     R12,#-1223  > 8
        SRP     PRIMREG
        IRET
;;
PL90S1: LD      R9,#PL90S2 & 255
        JP      @RR2
;;
PL90S2:
        LD      R9,#PL90S3 & 255
        ADD     R15,# 1982  & 255
        ADC     R14,# 1982  > 8
        ADD     R13,#-622  & 255
        ADC     R12,#-622  > 8
        SRP     PRIMREG
        IRET
;;
PL90S3:
        LD      R9,#PL90S4 & 255
        JP      @RR2
;;
PL90S4:
        LD      R9,#PL90S5 & 255
        ADD     R15,# 2286  & 255
        ADC     R14,# 2286  > 8
        ADD     R13,# 322  & 255
        ADC     R12,# 322  > 8
        SRP     PRIMREG
        IRET
;;
PL90S5: 
        LD      R9,#PL90S6 & 255
        JP      @RR2
;;
PL90S6:
        LD      R9,#PL90S7 & 255
        ADD     R15,# 2289  & 255
        ADC     R14,# 2289  > 8
        ADD     R13,# 1523  & 255
        ADC     R12,# 1523  > 8
        SRP     PRIMREG
        IRET
;;
PL90S7: 
        LD      R9,#PL90S8 & 255
        JP      @RR2
;;
PL90S8:
        LD      R9,#PL90S9 & 255
        ADD     R15,# 1523  & 255
        ADC     R14,# 1523  > 8
        ADD     R13,# 2289  & 255
        ADC     R12,# 2289  > 8
        SRP     PRIMREG
        IRET
;;
PL90S9:
        LD      R9,#PL90SA & 255
        JP      @RR2
;;
PL90SA:
        LD      R9,#PL90SB & 255
        ADD     R15,# 322  & 255
        ADC     R14,# 322  > 8
        ADD     R13,# 2286  & 255
        ADC     R12,# 2286  > 8
        SRP     PRIMREG
        IRET
;;
PL90SB:
        LD      R9,#PL90SC & 255
        JP      @RR2
;;
PL90SC:
        LD      R9,#PL90SD & 255
        ADD     R15,#-622  & 255
        ADC     R14,#-622  > 8
        ADD     R13,# 1982  & 255
        ADC     R12,# 1982  > 8
        SRP     PRIMREG
        IRET
;;
PL90SD:
        LD      R9,#PL90SE & 255
        JP      @RR2
;;
PL90SE:
        LD      R9,#PL90SF & 255
        ADD     R15,#-1223  & 255
        ADC     R14,#-1223  > 8
        ADD     R13,# 1635  & 255
        ADC     R12,# 1635  > 8
        SRP     PRIMREG
        IRET
;;
PL90SF:
        LD      R9,#PL90S0 & 255
        JP      @RR2
;;----------------------------------------------------------------
;;HERE ARE THE ZERO AND 180 DEGREE PHASE SHIFT ROUTINES
;;
ZERO:   LD      R9,#ZEROB & 255  ;;ZERO DEGREE PHASE SHIFT FOR EVEN STATES
        SRP     PRIMREG
        IRET
;;
ZEROB:  LD      R9,#ZERO & 255  ;;ZERO DEGREE PHASE SHIFT FOR ODD STATES
        JP      @RR2
;;
;;--------------------------------------------------------------------
;;
PLUS180:                        ;;180 DEGREE PHASE SHIFT (EVEN STATE TIMES)
        ADD     R14,#8          ;;1/8TH OF 180 DEGREES
        SUB     R12,#8
        LD      R9,#PLUS180B & 255
        SRP     PRIMREG
        IRET
;;
PLUS180B:                       ;;180 DEGREE ROUTINE FOR ODD STATES (NOP)
        LD      R9,#PLUS180 & 255
        JP      @RR2    
;;--------------------------------------------------------------------
;;THESE HARDWARE UPDATE OR REFRESH ROUTINES USE R0,R1,R2,TXHOLD.
;;NOTE: "TXHOLD" IS USED ONLY AS A GP REGISTER HERE.
;;
UD3435: CALL    UPDATE35        ;;UPDATE BOTH 34 AND 35
UPDATE34:                       ;;WRITE IMAGE34 TO HARDWAARE
        LD      R0,IMAGE34
        LD      TXHOLD,#"10H
        JR      UPDATEA
;;
UPDATE35:                       ;;WRITE IMAGE35 TO HARDWARE
        LD      R0,IMAGE35
        LD      TXHOLD,#"20H
UPDATEA:LD      R1,#70H         ;;FIRST ADDRESS
UPDATEB:CP      R1,#60H         ;;CARRIER DETECT ADDRESS?
        JR      EQ,UPDATED      ;;DONT UPDATE CARRIER DETECT STATUS!
        LD      R2,R0           ;;GET BIT TO OUTPUT FROM R0
        AND     R2,#80H
        OR      R2,R1           ;;COMBINE WITH ADDRESS
        PUSH    IMR             ;;SAVE INTERRUPT MASK
        DI
        LD      PORT0,R2        ;;OUTPUT ADDRESS AND DATA TO THE LATCH
        NOP                     ;;DELAY
        AND     PORT3,TXHOLD    ;;BRING STROBE LOW
        COM     TXHOLD  
        OR      PORT3,TXHOLD    ;;THEN HIGH AGAIN
        POP     IMR             ;;RESTORE IMR
        COM     TXHOLD
        LD      R2,#42          ;;WAIT 500 CPU CYCLES TO KEEP FROM DISABLING
UPDATEC:DJNZ    R2,UPDATEC      ;; / ... INTERRUPTS TOO OFTEN
UPDATED:RL      R0              ;;GET NEXT BIT
        SUB     R1,#10H         ;;NEXT ADDRESS
        JR      PL,UPDATEB
        RET
;;  **CAUTION, DO NOT CROSS NEXT PAGE BOUNDRY**
;;
;;----------------------------------------------------------------------
;;
        ORG     0F00H           ;;<---------- NEW PAGE ------<<<
;;
;;
;;MINUS 90 DEGREE PHASE SHIFT ROUTINES
MI90S0:
        LD      R9,#MI90S1 & 255
        SUB     R15,# 1635  & 255
        SBC     R14,# 1635  > 8
        SUB     R13,#-1223  & 255
        SBC     R12,#-1223  > 8
        SRP     PRIMREG
        IRET
;;
MI90S1:
        LD      R9,#MI90S2 & 255
        JP      @RR2
;;
MI90S2:
        LD      R9,#MI90S3 & 255
        SUB     R15,# 1982  & 255
        SBC     R14,# 1982  > 8
        SUB     R13,#-622  & 255
        SBC     R12,#-622  > 8
        SRP     PRIMREG
        IRET
;;
MI90S3: LD      R9,#MI90S4 & 255
        JP      @RR2
;;
MI90S4:
        LD      R9,#MI90S5 & 255
        SUB     R15,# 2286  & 255
        SBC     R14,# 2286  > 8
        SUB     R13,# 322  & 255
        SBC     R12,# 322  > 8
        SRP     PRIMREG
        IRET
;;
MI90S5: LD      R9,#MI90S6 & 255
        JP      @RR2
;;
MI90S6:
        LD      R9,#MI90S7 & 255
        SUB     R15,# 2289  & 255
        SBC     R14,# 2289  > 8
        SUB     R13,# 1523  & 255
        SBC     R12,# 1523  > 8
        SRP     PRIMREG
        IRET
;;
MI90S7:
        LD      R9,#MI90S8 & 255
        JP      @RR2
;;
MI90S8: 
        LD      R9,#MI90S9 & 255
        SUB     R15,# 1523  & 255
        SBC     R14,# 1523  > 8
        SUB     R13,# 2289  & 255
        SBC     R12,# 2289  > 8
        SRP     PRIMREG
        IRET
;;
MI90S9:
        LD      R9,#MI90SA & 255
        JP      @RR2
;;
MI90SA:
        LD      R9,#MI90SB & 255
        SUB     R15,# 322  & 255
        SBC     R14,# 322  > 8
        SUB     R13,# 2286  & 255
        SBC     R12,# 2286  > 8
        SRP     PRIMREG
        IRET
;;
MI90SB:
        LD      R9,#MI90SC & 255
        JP      @RR2
;;
MI90SC:
        LD      R9,#MI90SD & 255
        SUB     R15,#-622  & 255
        SBC     R14,#-622  > 8
        SUB     R13,# 1982  & 255
        SBC     R12,# 1982  > 8
        SRP     PRIMREG
        IRET
;;
MI90SD:
        LD      R9,#MI90SE & 255
        JP      @RR2
;;
MI90SE:
        LD      R9,#MI90SF & 255
        SUB     R15,#-1223  & 255
        SBC     R14,#-1223  > 8
        SUB     R13,# 1635  & 255
        SBC     R12,# 1635  > 8
        SRP     PRIMREG
        IRET
;;
MI90SF:
        LD      R9,MI90S0 & 255
        JP      @RR2
;;
;;------------------------------------------------------
;;THIS IS THE COMMAND VECTOR TABLE
;;
CMDTBL:
        DB    'D'             ;;DIAL;;
        DW    DIAL
        DB    '.'
        DW    DIT
        DB    '-'
        DW    DAH
        DB    ':'             ;;GLOBAL SET COMMAND;;
        DW    ANYLOC
CMDT2:  DB    'M'             ;;MONITOR SPEAKER CONTROL;;
        DW    SPCRL
        DB    '/'
        DW    W120
        DB    'H'             ;;SWITCH HOOK CONTROL;;
        DW    HOOKCRL
        DB    'A'               ;;ANSWER PHONE WITHOUT RINGS
        DW    A$COMM
        DB    'F'             ;;FULL DUPLEX;;
        DW    SETFULL
        DB    'C'             ;;CARRIER ON/OFF ;;
        DW    SETCARRIER
        DB    'E'             ;;LOCAL ECHO ON/OFF;;
        DW    SETECHO
        DB    'R'             ;;REVERSE ORIG./ANSW. MODES;;
        DW    REVERSE
        DB    'S'             ;;DIRECT SET COMMAND;;
        DW    SETADR
        DB    '='             ;;DATA PART OF SET COMMAND;;
        DW    SETRAM
        DB    '?'             ;;GET CONTENTS OF ADDRESS;;
        DW    GETRAM
        DB    'V'             ;;VERBOSE MODE;;
        DW    SETVERBOSE
        DB    'O'             ;;GO ONLINE;;
        DW    GO$ONLINE
        DB    'Z'             ;;SOFT RESET;;
        DW    RESTART
        DB    'Q'             ;;QUIET MODE (NO RESULT CODES);;
        DW    SETQUIET
        DB    ','             ;;PAUSE;;
        DW    WAIT2SEC
        DB    'T'               ;;TOUCH TONE ENABLE
        DW    ENABTONE
        DB    'P'               ;;PULSE DIAL ENABLE
        DW    ENABPULSE
        DB      'I'             ;;PRODUCT ID COMMAND
        DW      PRODUCT$ID
        DB      'X'             ;;EXTENDED RESULT CODE ENABLE
        DW      EXRESULT
        DB      ';'             ;;TERMINATE DIAL COMMAND
        DW      STOP$DIAL
        DB      0               ;;END OF TABLE;;
;;
;;---------THIS IS THE "X" COMMAND--------------------------
;;
EXRESULT:
        AND     USTAT,#"EXR             ;;ASSUME NORMAL RESULT CODES
        CALL    NEXTNUM
        JR      Z,EXRDONE               ;;EXIT NOW IF "X0"
        OR      USTAT,#EXR              ;;ELSE SET FOR EXTENDED RESULT CODES
EXRDONE:RET     
;;

        END
